@@ -7,7 +7,7 @@
 # SPDX-License-Identifier: GPL-3.0
 
 # TEST_LIMIT: Number of tests to run at a time.
-# TEST_PATTERN: A Regex (PCRE) for tests to run.
+# TEST_PATTERN: An Extended Regex for tests to run.
 test_pattern_set=false
 case "$1" in
     '')
@@ -37,22 +37,25 @@ esac
 SKIP_PATTERN='(backupfile|bison|copy-file|datetime|exception|exclude|gettext|link-warning|localename-environ|mbsstr|nan|poll|printf|quotearg|trapping|vma-prot|wctype|windows|^(array|java|uni)|-ieee$|(^(execinfo|getloadavg|gnumakefile|havelib|physmem|timevar)$))'
 
 # Tests that have failed in the past.
-# copy-file fails to compile on i586 so we skip it.
+# canonicalize-lgpl: https://builds.sr.ht/~alip/job/1549072#task-test64
 # linkat: https://builds.sr.ht/query/log/1402429/test32/log
 # mkdir: https://builds.sr.ht/~alip/job/1286254
 # openat-safer: https://builds.sr.ht/~alip/job/1522333
 # posix_openpt: https://builds.sr.ht/~alip/job/126488
 # readdir: no known fail but best to check everytime.
 # rename: https://builds.sr.ht/~alip/job/1286933
+# select: https://builds.sr.ht/~alip/job/1574787
 # truncate: https://gitlab.exherbo.org/sydbox/sydbox/-/jobs/83046
 # utimensat: https://builds.sr.ht/~alip/job/1257729
 FAIL_HISTORY=(
+    canonicalize-lgpl
     linkat
     mkdir
     openat-safer
     posix_openpt
     readdir
     rename
+    select
     truncate
     utimensat
 )
@@ -68,15 +71,16 @@ ulimit -c unlimited
 # Force TTY output.
 export SYD_FORCE_TTY=YesPlease
 
-# Timeout is 15 minutes per-test,
+# Timeout is 20 minutes per-test,
 # unless otherwise specified.
-SYD_TEST_TIMEOUT=${SYD_TEST_TIMEOUT:-15m}
+SYD_TEST_TIMEOUT=${SYD_TEST_TIMEOUT:-20m}
 
 export SYD_LOG=${SYD_LOG:-info}
 SYD="${CARGO_BIN_EXE_syd:-syd}"
 
 set -ex
-DIR="$(mktemp -d --tmpdir=/tmp syd-gnulib.XXXXX)"
+DIR="$(mktemp -d syd-gnulib.XXXXXX)"
+DIR="$(readlink -f "${DIR}")"
 set +ex
 
 edo() {
@@ -156,7 +160,7 @@ set +x
 PASS=0
 FAIL=0
 SKIP=0
-TESTS=( $(./gnulib-tool --list | grep -P "${TEST_PATTERN}" | grep -vP "${SKIP_PATTERN}" | shuf ) )
+TESTS=( $(./gnulib-tool --list | grep -E "${TEST_PATTERN}" | grep -vE "${SKIP_PATTERN}" | shuf ) )
 CTEST=${#TESTS[@]}
 NTEST=${TEST_LIMIT}
 if [[ ${NTEST} -gt ${CTEST} ]]; then
@@ -169,7 +173,7 @@ idx=0
 for name in "${TESTS[@]}"; do
     : $(( idx++ ))
     echo >&2 -e "\033[92m*** $name ($idx of $NTEST: $PASS ok, $FAIL notok, $SKIP todo) ***\033[0m"
-    if echo "${name}" | grep -qP "${SKIP_PATTERN}"; then
+    if echo "${name}" | grep -qE "${SKIP_PATTERN}"; then
         echo "ok ${idx} - ${name} # TODO"
         : $(( SKIP++ ))
     elif run_test "${name}"; then
