@@ -17,6 +17,7 @@ use std::{
     cmp::Ordering,
     os::fd::{AsFd, AsRawFd, RawFd},
     ptr::NonNull,
+    sync::LazyLock,
 };
 
 use bitflags::bitflags;
@@ -33,59 +34,35 @@ use nix::{
     unistd::Pid,
     NixPath,
 };
-use once_cell::sync::Lazy;
 use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{config::*, fs::FileType, XPath};
 
+// x32 compatibility
+// See https://sourceware.org/bugzilla/show_bug.cgi?id=16437
+#[cfg(all(target_arch = "x86_64", target_pointer_width = "32"))]
+#[expect(non_camel_case_types)]
+pub(crate) type timespec_tv_nsec_t = i64;
+#[cfg(not(all(target_arch = "x86_64", target_pointer_width = "32")))]
+#[expect(non_camel_case_types)]
+pub(crate) type timespec_tv_nsec_t = libc::c_long;
+
+// C-compatible layout of the `timespec64` structure.
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 #[repr(C)]
 pub(crate) struct TimeSpec64 {
     pub(crate) tv_sec: i64,
     pub(crate) tv_nsec: i64,
 }
 
+// C-compatible layout of the `timespec32` structure.
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 #[repr(C)]
 pub(crate) struct TimeSpec32 {
     pub(crate) tv_sec: i32,
     pub(crate) tv_nsec: i32,
 }
 
-/// Replacement for slice::trim_ascii_start which is Rust>=1.80.0.
-#[inline]
-pub const fn trim_ascii_start(mut bytes: &[u8]) -> &[u8] {
-    // Note: A pattern matching based approach (instead of indexing)
-    // allows making the function const.
-    while let [first, rest @ ..] = bytes {
-        if first.is_ascii_whitespace() {
-            bytes = rest;
-        } else {
-            break;
-        }
-    }
-    bytes
-}
-
-/// Replacement for slice::trim_ascii_end which is Rust>=1.80.0.
-#[inline]
-pub const fn trim_ascii_end(mut bytes: &[u8]) -> &[u8] {
-    // Note: A pattern matching based approach (instead of indexing)
-    // allows making the function const.
-    while let [rest @ .., last] = bytes {
-        if last.is_ascii_whitespace() {
-            bytes = rest;
-        } else {
-            break;
-        }
-    }
-    bytes
-}
-
-/// Replacement for slice::trim_ascii which is Rust>=1.80.0.
-#[inline]
-pub const fn trim_ascii(bytes: &[u8]) -> &[u8] {
-    trim_ascii_end(trim_ascii_start(bytes))
-}
-
 /// Aligns the given length to the nearest 4-byte boundary.
 ///
 /// This function is useful for ensuring that data structures in a 32-bit architecture
@@ -193,8 +170,8 @@ pub struct stat32 {
 }
 
 impl From<libc::stat64> for stat32 {
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::unnecessary_cast)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::unnecessary_cast)]
     fn from(stat: libc::stat64) -> Self {
         Self {
             st_dev: stat.st_dev as u64,
@@ -220,6 +197,7 @@ impl From<libc::stat64> for stat32 {
 }
 
 /// This structure represents the Linux 32-bit data structure 'struct iovec'
+#[derive(Copy, Clone)]
 #[repr(C)]
 pub struct iovec32 {
     iov_base: u32,
@@ -269,6 +247,52 @@ impl From<mmsghdr> for mmsghdr32 {
     }
 }
 
+const _: () = {
+    assert!(
+        size_of::<libc::mmsghdr>() == size_of::<mmsghdr>(),
+        "Size mismatch between libc::mmsghdr and compat::mmsghdr"
+    );
+};
+
+impl From<libc::mmsghdr> for mmsghdr {
+    fn from(msg: libc::mmsghdr) -> Self {
+        // SAFETY: Since we have confirmed the sizes are the same at compile time,
+        // we can safely perform a transmute. This assumes no differences in alignment
+        // or memory layout between libc::mmsghdr and mmsghdr.
+        unsafe { std::mem::transmute(msg) }
+    }
+}
+
+impl From<libc::mmsghdr> for mmsghdr32 {
+    fn from(msg: libc::mmsghdr) -> Self {
+        mmsghdr::from(msg).into()
+    }
+}
+
+impl From<mmsghdr> for libc::mmsghdr {
+    fn from(msg: mmsghdr) -> Self {
+        // SAFETY: Since we have confirmed the sizes are the same at compile time,
+        // we can safely perform a transmute. This assumes no differences in alignment
+        // or memory layout between libc::mmsghdr and mmsghdr.
+        unsafe { std::mem::transmute(msg) }
+    }
+}
+
+impl From<mmsghdr32> for libc::mmsghdr {
+    fn from(msg: mmsghdr32) -> Self {
+        mmsghdr::from(msg).into()
+    }
+}
+
+/// This union represents the Linux data structure 'struct mmsghdr'
+#[repr(C)]
+pub union mmsghdr_union {
+    // 32-bit mmsghdr32
+    pub m32: mmsghdr32,
+    // Native mmsghdr
+    pub m64: mmsghdr,
+}
+
 /// This structure represents the Linux 32-bit data structure 'struct cmsghdr'
 #[repr(C)]
 pub struct cmsghdr32 {
@@ -336,8 +360,8 @@ impl From<msghdr32> for msghdr {
     }
 }
 
-#[allow(clippy::unnecessary_cast)]
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::unnecessary_cast)]
+#[expect(clippy::cast_possible_truncation)]
 impl From<msghdr> for msghdr32 {
     fn from(msg: msghdr) -> Self {
         msghdr32 {
@@ -352,6 +376,28 @@ impl From<msghdr> for msghdr32 {
     }
 }
 
+const _: () = {
+    assert!(
+        size_of::<libc::msghdr>() == size_of::<msghdr>(),
+        "Size mismatch between libc::msghdr and compat::msghdr"
+    );
+};
+
+impl From<libc::msghdr> for msghdr {
+    fn from(msg: libc::msghdr) -> Self {
+        // SAFETY: Since we have confirmed the sizes are the same at compile time,
+        // we can safely perform a transmute. This assumes no differences in alignment
+        // or memory layout between libc::msghdr and msghdr.
+        unsafe { std::mem::transmute(msg) }
+    }
+}
+
+impl From<libc::msghdr> for msghdr32 {
+    fn from(msg: libc::msghdr) -> Self {
+        msghdr::from(msg).into()
+    }
+}
+
 /// Rust equivalent of the Linux kernel's struct xattr_args:
 #[repr(C)]
 #[derive(Debug, Copy, Clone)]
@@ -370,7 +416,7 @@ pub struct XattrArgs {
 // Note getxattrat may not be available,
 // and libc::SYS_getxattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_GETXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_GETXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("getxattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -398,8 +444,8 @@ pub unsafe fn getxattrat<Fd: AsFd, P: ?Sized + NixPath>(
     };
 
     path.with_nix_path(|c_path| {
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
         Errno::result(
             // SAFETY: In libc we trust.
             unsafe {
@@ -421,7 +467,7 @@ pub unsafe fn getxattrat<Fd: AsFd, P: ?Sized + NixPath>(
 // Note setxattrat may not be available,
 // and libc::SYS_setxattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_SETXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_SETXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("setxattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -468,7 +514,7 @@ pub unsafe fn setxattrat<Fd: AsFd, P: ?Sized + NixPath>(
 // Note listxattrat may not be available,
 // and libc::SYS_listxattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_LISTXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_LISTXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("listxattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -496,8 +542,8 @@ pub unsafe fn listxattrat<Fd: AsFd, P: ?Sized + NixPath>(
     };
 
     path.with_nix_path(|c_path| {
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
         Errno::result(
             // SAFETY: In libc we trust.
             unsafe {
@@ -518,7 +564,7 @@ pub unsafe fn listxattrat<Fd: AsFd, P: ?Sized + NixPath>(
 // Note removexattrat may not be available,
 // and libc::SYS_removexattrat may not be defined.
 // Therefore we query the number using libseccomp.
-static SYS_REMOVEXATTRAT: Lazy<libc::c_long> = Lazy::new(|| {
+static SYS_REMOVEXATTRAT: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("removexattrat")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -633,7 +679,7 @@ pub struct FileStatx {
     pub stx_rdev_minor: u32,
 
     // Note, these are not not public on purpose
-    // as they return inconsistent values on filesytems
+    // as they return inconsistent values on filesystems
     // such as btrfs and overlayfs. `stx_mnt_id` should
     // be used instead.
     pub(crate) stx_dev_major: u32, // ID of device containing file [uncond]
@@ -643,7 +689,18 @@ pub struct FileStatx {
     stx_dio_mem_align: u32,    // Memory buffer alignment for direct I/O
     stx_dio_offset_align: u32, // File offset alignment for direct I/O
 
-    __statx_pad2: [u64; 12], // Spare space for future expansion
+    stx_subvol: u64, // Subvolume identifier
+
+    stx_atomic_write_unit_min: u32, // Min atomic write unit in bytes
+    stx_atomic_write_unit_max: u32, // Max atomic write unit in bytes
+    stx_atomic_write_segments_max: u32, // Max atomic write segment count
+
+    stx_dio_read_offset_align: u32, // File offset alignment for direct I/O reads
+
+    stx_atomic_write_unit_max_opt: u32, // Optimised max atomic write unit in bytes
+    __statx_spare2: [u32; 1],
+
+    __statx_spare3: [u64; 8], // Spare space for future expansion
 }
 
 impl FileStatx {
@@ -761,7 +818,7 @@ impl DirIter {
 impl<'a> Iterator for &'a mut DirIter {
     type Item = DirEntry<'a>;
 
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     fn next(&mut self) -> Option<Self::Item> {
         if self.offset >= self.bufsiz {
             return None;
@@ -769,7 +826,7 @@ impl<'a> Iterator for &'a mut DirIter {
 
         // SAFETY: Parse the next dirent safely by borrowing from the buffer.
         unsafe {
-            #[allow(clippy::cast_ptr_alignment)]
+            #[expect(clippy::cast_ptr_alignment)]
             let dirent_ptr = self
                 .buffer
                 .as_ptr()
@@ -791,7 +848,7 @@ impl<'a> Iterator for &'a mut DirIter {
 
 impl Drop for DirIter {
     fn drop(&mut self) {
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let layout = Layout::from_size_align(self.memsiz, DIRENT_ALIGN).unwrap();
 
         // SAFETY: Deallocate buffer.
@@ -997,8 +1054,8 @@ pub fn getdents64<Fd: AsFd>(
 }
 
 /// Wrapper for the `getdents64` syscall.
-#[allow(clippy::cast_possible_truncation)]
-#[allow(clippy::cast_sign_loss)]
+#[expect(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_sign_loss)]
 fn sys_getdents64<Fd: AsFd>(fd: Fd, buf: *mut libc::c_void, bytes: usize) -> Result<usize, Errno> {
     // SAFETY: In kernel, we trust.
     Errno::result(unsafe {
@@ -1106,7 +1163,7 @@ impl WaitStatus {
 /// Wrapper for the `waitid` syscall
 /// This is identical to nix' waitid except we use our custom WaitStatus.
 pub fn waitid(id: Id, flags: WaitPidFlag) -> Result<WaitStatus, Errno> {
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_sign_loss)]
     let (idtype, idval) = match id {
         Id::All => (libc::P_ALL, 0),
         Id::Pid(pid) => (libc::P_PID, pid.as_raw() as libc::id_t),
@@ -1120,7 +1177,7 @@ pub fn waitid(id: Id, flags: WaitPidFlag) -> Result<WaitStatus, Errno> {
         // Memory is zeroed rather than uninitialized, as not all platforms
         // initialize the memory in the StillAlive case
         let mut siginfo: libc::siginfo_t = std::mem::zeroed();
-        Errno::result(libc::waitid(idtype, idval, &mut siginfo, flags.bits()))?;
+        Errno::result(libc::waitid(idtype, idval, &raw mut siginfo, flags.bits()))?;
         siginfo
     };
 
@@ -1171,17 +1228,17 @@ pub(crate) fn pipe2_raw(flags: OFlag) -> Result<(RawFd, RawFd), Errno> {
     Ok((read, write))
 }
 
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub(crate) const PF_UNSPEC: libc::sa_family_t = libc::AF_UNSPEC as libc::sa_family_t;
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub(crate) const PF_UNIX: libc::sa_family_t = libc::AF_UNIX as libc::sa_family_t;
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub(crate) const PF_INET: libc::sa_family_t = libc::AF_INET as libc::sa_family_t;
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub(crate) const PF_INET6: libc::sa_family_t = libc::AF_INET6 as libc::sa_family_t;
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub(crate) const PF_ALG: libc::sa_family_t = libc::AF_ALG as libc::sa_family_t;
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::cast_possible_truncation)]
 pub(crate) const PF_NETLINK: libc::sa_family_t = libc::AF_NETLINK as libc::sa_family_t;
 /* From <bits/socket.h>, expect this to be updated regularly. */
 pub(crate) const PF_MAX: libc::sa_family_t = 46;
@@ -1203,18 +1260,34 @@ pub const STATX_NLINK: libc::c_uint = 0x00000004;
 pub const STATX_UID: libc::c_uint = 0x00000008;
 /// Want/got stx_gid.
 pub const STATX_GID: libc::c_uint = 0x00000010;
+/// Want/got stx_atime.
+pub const STATX_ATIME: libc::c_uint = 0x00000020;
+/// Want/got stx_mtime.
+pub const STATX_MTIME: libc::c_uint = 0x00000040;
 /// Want/got stx_ctime.
 pub const STATX_CTIME: libc::c_uint = 0x00000080;
 /// Want/got stx_ino.
 pub const STATX_INO: libc::c_uint = 0x00000100;
 /// Want/got stx_size.
 pub const STATX_SIZE: libc::c_uint = 0x00000200;
-/// Want/Got stx_mnt_id.
-pub const STATX_MNT_ID: libc::c_uint = 0x00001000;
-/// Want/got extended stx_mount_id, requires Linux>=6.8.
-pub const STATX_MNT_ID_UNIQUE: libc::c_uint = 0x00004000;
+/// Want/got stx_blocks.
+pub const STATX_BLOCKS: libc::c_uint = 0x00000400;
 /// Want all the basic stat information.
 pub const STATX_BASIC_STATS: libc::c_uint = 0x000007ff;
+/// Want/got stx_btime.
+pub const STATX_BTIME: libc::c_uint = 0x00000800;
+/// Want/Got stx_mnt_id.
+pub const STATX_MNT_ID: libc::c_uint = 0x00001000;
+/// Want/got direct I/O alignment info.
+pub const STATX_DIOALIGN: libc::c_uint = 0x00002000;
+/// Want/got extended stx_mount_id, requires Linux>=6.8.
+pub const STATX_MNT_ID_UNIQUE: libc::c_uint = 0x00004000;
+/// Want/got stx_subvol.
+pub const STATX_SUBVOL: libc::c_uint = 0x00008000;
+/// Want/got atomic_write_* fields.
+pub const STATX_WRITE_ATOMIC: libc::c_uint = 0x00010000;
+/// Want/got dio read alignment info.
+pub const STATX_DIO_READ_ALIGN: libc::c_uint = 0x00020000;
 
 /// Do what stat(2) does, default.
 pub const AT_STATX_SYNC_AS_STAT: libc::c_int = 0x0000;
@@ -1316,7 +1389,9 @@ type fs_type_t = libc::c_int;
 type fs_type_t = libc::__fsword_t;
 
 const BTRFS_SUPER_MAGIC: fs_type_t = libc::BTRFS_SUPER_MAGIC as fs_type_t;
+const HUGETLBFS_MAGIC: fs_type_t = libc::HUGETLBFS_MAGIC as fs_type_t;
 const OVERLAYFS_SUPER_MAGIC: fs_type_t = libc::OVERLAYFS_SUPER_MAGIC as fs_type_t;
+const PROC_SUPER_MAGIC: fs_type_t = libc::PROC_SUPER_MAGIC as fs_type_t;
 
 /// Wrapper for struct statfs64
 pub(crate) struct Statfs64(libc::statfs64);
@@ -1333,6 +1408,16 @@ impl Statfs64 {
     pub(crate) fn has_broken_device_ids(&self) -> bool {
         matches!(self.0.f_type, OVERLAYFS_SUPER_MAGIC | BTRFS_SUPER_MAGIC)
     }
+
+    // Check if file resides on a hugetlbfs.
+    pub(crate) fn is_huge_file(&self) -> bool {
+        self.0.f_type == HUGETLBFS_MAGIC
+    }
+
+    // Check if file resides on a procfs.
+    pub(crate) fn is_proc(&self) -> bool {
+        self.0.f_type == PROC_SUPER_MAGIC
+    }
 }
 
 /// Safe wrapper for fstatfs64
@@ -1361,7 +1446,7 @@ pub fn epoll_ctl_safe<E: AsFd>(
                     epoll.as_fd().as_raw_fd(),
                     EpollOp::EpollCtlAdd as libc::c_int,
                     fd,
-                    &mut event,
+                    &raw mut event,
                 )
             }),
             Errno::EEXIST,
@@ -1400,7 +1485,7 @@ pub fn epoll_ctl_mod_safe<E: AsFd>(
             epoll.as_fd().as_raw_fd(),
             EpollOp::EpollCtlMod as libc::c_int,
             fd,
-            &mut event,
+            &raw mut event,
         )
     })
     .map(drop)
@@ -1492,7 +1577,7 @@ pub fn epoll_get_params<Fd: AsFd>(fd: Fd) -> Result<EpollParams, Errno> {
 
 /// Uses getsockopt SO_DOMAIN to get the domain of the given socket.
 pub fn getsockdomain<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     let mut len = std::mem::size_of::<libc::c_int>() as libc::socklen_t;
     let mut fml: libc::c_int = 0;
 
@@ -1503,7 +1588,7 @@ pub fn getsockdomain<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
             libc::SOL_SOCKET,
             libc::SO_DOMAIN,
             std::ptr::addr_of_mut!(fml) as *mut _,
-            &mut len,
+            &raw mut len,
         )
     })?;
 
@@ -1577,8 +1662,8 @@ impl std::fmt::Display for LsmId {
     }
 }
 
-/// Lazy-initialized system call number for `lsm_list_modules`.
-pub static SYS_LSM_LIST_MODULES: Lazy<libc::c_long> = Lazy::new(|| {
+/// LazyLock-initialized system call number for `lsm_list_modules`.
+pub static SYS_LSM_LIST_MODULES: LazyLock<libc::c_long> = LazyLock::new(|| {
     ScmpSyscall::from_name("lsm_list_modules")
         .map(i32::from)
         .map(libc::c_long::from)
@@ -1619,8 +1704,8 @@ pub fn lsm_list_modules() -> Result<Vec<LsmId>, Errno> {
 
     let count = (size / 8) as usize;
     let mut buf = vec![0u64; count];
-    #[allow(clippy::cast_possible_truncation)]
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_sign_loss)]
     let count = Errno::result(
         // SAFETY: Properly allocated buffer & valid pointer to `size`.
         unsafe {
@@ -1653,7 +1738,7 @@ bitflags! {
     #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
     #[repr(transparent)]
     pub struct RenameFlags: u32 {
-        /// Don’t overwrite `new_path` of the rename;
+        /// Don't overwrite `new_path` of the rename;
         /// return an error if `new_path` already exists.
         const RENAME_NOREPLACE = 1;
 
@@ -1661,8 +1746,263 @@ bitflags! {
         /// Both paths must exist.
         const RENAME_EXCHANGE  = 2;
 
-        /// Create a “whiteout” at the source of the rename
+        /// Create a "whiteout" at the source of the rename
         /// (for overlay/union filesystems).
         const RENAME_WHITEOUT  = 4;
     }
 }
+
+bitflags! {
+    /// Flags used with `send`, `recv`, etc.
+    // Keep in sync with <linux/socket.h>!
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct MsgFlags: i32 {
+        /// Process out-of-band data.
+        const MSG_OOB = 0x01;
+        /// Peek at incoming messages.
+        const MSG_PEEK = 0x02;
+        /// Don't use local routing.
+        const MSG_DONTROUTE = 0x04;
+        /// DECnet uses a different name for MSG_DONTROUTE.
+        const MSG_TRYHARD = Self::MSG_DONTROUTE.bits();
+        /// Control data lost before delivery.
+        const MSG_CTRUNC = 0x08;
+        /// Supply or ask second address.
+        const MSG_PROXY = 0x10;
+        /// Truncated message.
+        const MSG_TRUNC = 0x20;
+        /// Nonblocking IO.
+        const MSG_DONTWAIT = 0x40;
+        /// End of record.
+        const MSG_EOR = 0x80;
+        /// Wait for a full request.
+        const MSG_WAITALL = 0x100;
+        /// End of connection.
+        const MSG_FIN = 0x200;
+        /// Synchronous operation.
+        const MSG_SYN = 0x400;
+        /// Confirm path validity.
+        const MSG_CONFIRM = 0x800;
+        /// Reset connection.
+        const MSG_RST = 0x1000;
+        /// Fetch message from error queue.
+        const MSG_ERRQUEUE = 0x2000;
+        /// Do not generate SIGPIPE.
+        const MSG_NOSIGNAL = 0x4000;
+        /// Sender will send more data.
+        const MSG_MORE = 0x8000;
+        /// Wait for at least one packet to return.
+        const MSG_WAITFORONE = 0x10000;
+        /// More messages coming.
+        const MSG_BATCH = 0x40000;
+        /// Receive devmem skbs as cmsg.
+        const MSG_SOCK_DEVMEM = 0x2000000;
+        /// Use user data in kernel path.
+        const MSG_ZEROCOPY = 0x4000000;
+        /// Send data in TCP SYN.
+        const MSG_FASTOPEN = 0x20000000;
+        /// Set close_on_exit for file descriptors.
+        const MSG_CMSG_CLOEXEC = 0x40000000;
+        /// Special flag for notifications.
+        const MSG_NOTIFICATION = Self::MSG_MORE.bits();
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+use nix::sys::socket::MsgFlags as NixMsgFlags;
+
+#[expect(clippy::disallowed_types)]
+impl From<MsgFlags> for NixMsgFlags {
+    fn from(msgflags: MsgFlags) -> Self {
+        Self::from_bits_retain(msgflags.bits())
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+impl From<NixMsgFlags> for MsgFlags {
+    fn from(msgflags: NixMsgFlags) -> Self {
+        Self::from_bits_retain(msgflags.bits())
+    }
+}
+
+bitflags! {
+    /// Flags used with inotify_add_watch(2).
+    // Keep in sync with <linux/inotify.h>!
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct AddWatchFlags: u32 {
+        // The following are legal, implemented events that user-space can watch for.
+        /// File was accessed.
+        const IN_ACCESS = 0x00000001;
+        /// File was modified.
+        const IN_MODIFY = 0x00000002;
+        /// Metadata changed.
+        const IN_ATTRIB = 0x00000004;
+        /// Writable file was closed.
+        const IN_CLOSE_WRITE = 0x00000008;
+        /// Unwritable file closed.
+        const IN_CLOSE_NOWRITE = 0x00000010;
+        /// File was opened.
+        const IN_OPEN = 0x00000020;
+        /// File was moved from X.
+        const IN_MOVED_FROM = 0x00000040;
+        /// File was moved to Y.
+        const IN_MOVED_TO = 0x00000080;
+        /// Subfile was created.
+        const IN_CREATE = 0x00000100;
+        /// Subfile was deleted.
+        const IN_DELETE = 0x00000200;
+        /// Self was deleted.
+        const IN_DELETE_SELF = 0x00000400;
+        /// Self was moved.
+        const IN_MOVE_SELF = 0x00000800;
+
+        // The following are legal events. They are sent as needed to any watch.
+        /// Backing fs was unmounted.
+        const IN_UNMOUNT = 0x00002000;
+        /// Event queued overflowed.
+        const IN_Q_OVERFLOW = 0x00004000;
+        /// File was ignored.
+        const IN_IGNORED = 0x00008000;
+
+        // Helper events.
+        /// Helper event: Close.
+        const IN_CLOSE = Self::IN_CLOSE_WRITE.bits() | Self::IN_CLOSE_NOWRITE.bits();
+        /// Helper event: Moves.
+        const IN_MOVE = Self::IN_MOVED_FROM.bits() | Self::IN_MOVED_TO.bits();
+
+        // Special flags.
+        /// Only watch the path if it is a directory.
+        const IN_ONLYDIR = 0x01000000;
+        /// Don't follow a sym link.
+        const IN_DONT_FOLLOW = 0x02000000;
+        /// Exclude events on unlinked objects.
+        const IN_EXCL_UNLINK = 0x04000000;
+        /// Only create watches.
+        const IN_MASK_CREATE = 0x10000000;
+        /// Add to the mask of an already existing watch.
+        const IN_MASK_ADD = 0x20000000;
+        /// Event occurred against dir.
+        const IN_ISDIR = 0x40000000;
+        /// Only send event once.
+        const IN_ONESHOT = 0x80000000;
+
+        /*
+         * All of the events - we build the list by hand so that we can add flags in
+         * the future and not break backward compatibility. Apps will get only the
+         * events that they originally wanted. Be sure to add new events here!
+         */
+        const IN_ALL_EVENTS =
+            Self::IN_ACCESS.bits() |
+            Self::IN_MODIFY.bits() |
+            Self::IN_ATTRIB.bits() |
+            Self::IN_CLOSE_WRITE.bits() |
+            Self::IN_CLOSE_NOWRITE.bits() |
+            Self::IN_OPEN.bits() |
+            Self::IN_MOVED_FROM.bits() |
+            Self::IN_MOVED_TO.bits() |
+            Self::IN_DELETE.bits() |
+            Self::IN_CREATE.bits() |
+            Self::IN_DELETE_SELF.bits() |
+            Self::IN_MOVE_SELF.bits();
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+use nix::sys::inotify::AddWatchFlags as NixAddWatchFlags;
+
+#[expect(clippy::disallowed_types)]
+impl From<AddWatchFlags> for NixAddWatchFlags {
+    fn from(addwatchflags: AddWatchFlags) -> Self {
+        Self::from_bits_retain(addwatchflags.bits())
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+impl From<NixAddWatchFlags> for AddWatchFlags {
+    fn from(addwatchflags: NixAddWatchFlags) -> Self {
+        Self::from_bits_retain(addwatchflags.bits())
+    }
+}
+
+bitflags! {
+    /// Flags for memfd_create(2)
+    // nix' MFdFlags does not include MFD_{EXEC,NOEXEC_SEAL} yet!
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct MFdFlags: libc::c_uint {
+        /// Close-on-exec
+        const MFD_CLOEXEC = libc::MFD_CLOEXEC;
+
+        /// Allow sealing via _fcntl_(2).
+        const MFD_ALLOW_SEALING = libc::MFD_ALLOW_SEALING;
+
+        /// Disallow exec (Linux >= 6.3).
+        const MFD_NOEXEC_SEAL = libc::MFD_NOEXEC_SEAL;
+
+        /// Allow exec (Linux >= 6.3).
+        const MFD_EXEC = libc::MFD_EXEC;
+
+        /// Use hugetlbfs.
+        const MFD_HUGETLB = libc::MFD_HUGETLB;
+
+        /// Huge page size: 1MB.
+        const MFD_HUGE_1MB = libc::MFD_HUGE_1MB;
+
+        /// Huge page size: 2MB.
+        const MFD_HUGE_2MB = libc::MFD_HUGE_2MB;
+
+        /// Huge page size: 8MB.
+        const MFD_HUGE_8MB = libc::MFD_HUGE_8MB;
+
+        /// Huge page size: 16MB.
+        const MFD_HUGE_16MB = libc::MFD_HUGE_16MB;
+
+        /// Huge page size: 32MB.
+        const MFD_HUGE_32MB = libc::MFD_HUGE_32MB;
+
+        /// Huge page size: 256MB.
+        const MFD_HUGE_256MB = libc::MFD_HUGE_256MB;
+
+        /// Huge page size: 512MB.
+        const MFD_HUGE_512MB = libc::MFD_HUGE_512MB;
+
+        /// Huge page size: 1GB.
+        const MFD_HUGE_1GB = libc::MFD_HUGE_1GB;
+
+        /// Huge page size: 2GB.
+        const MFD_HUGE_2GB = libc::MFD_HUGE_2GB;
+
+        /// Huge page size: 16GB.
+        const MFD_HUGE_16GB = libc::MFD_HUGE_16GB;
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+use nix::sys::memfd::MFdFlags as NixMFdFlags;
+
+#[expect(clippy::disallowed_types)]
+impl From<MFdFlags> for NixMFdFlags {
+    fn from(mfdflags: MFdFlags) -> Self {
+        Self::from_bits_retain(mfdflags.bits())
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+impl From<NixMFdFlags> for MFdFlags {
+    fn from(mfdflags: NixMFdFlags) -> Self {
+        Self::from_bits_retain(mfdflags.bits())
+    }
+}
+
+/*
+ * Constants from <linux/limits.h> not defined by libc yet.
+ */
+
+// # chars in an extended attribute name.
+pub(crate) const XATTR_NAME_MAX: usize = 255;
+// size of an extended attribute value (64k).
+pub(crate) const XATTR_SIZE_MAX: usize = 1 << 16;
+// size of extended attribute namelist (64k).
+pub(crate) const XATTR_LIST_MAX: usize = 1 << 16;
