@@ -17,6 +17,11 @@
 
 // SAFETY: This module has been liberated from unsafe code!
 #![forbid(unsafe_code)]
+// SAFETY: This module has been liberated from arithmetic side effects!
+#![forbid(clippy::arithmetic_side_effects)]
+#![forbid(clippy::cast_possible_truncation)]
+#![forbid(clippy::cast_possible_wrap)]
+#![forbid(clippy::cast_sign_loss)]
 
 //! Set of functions to manage parsing ELF files
 
@@ -27,6 +32,7 @@ use std::{
     os::fd::AsFd,
 };
 
+use memchr::arch::all::{is_equal, is_prefix};
 use nix::errno::Errno;
 
 use crate::{fs::safe_open_file, XPath};
@@ -172,42 +178,56 @@ pub enum ElfError {
     IoError(io::Error),
 }
 
+impl From<u16> for ElfFileType {
+    fn from(e_type: u16) -> Self {
+        match e_type {
+            ET_NONE => Self::None,
+            ET_EXEC => Self::Executable,
+            ET_DYN => Self::Library,
+            ET_REL => Self::Relocatable,
+            ET_CORE => Self::Core,
+            ET_LOPROC..=ET_HIPROC => Self::ProcessorSpecific,
+            _ => Self::Unknown,
+        }
+    }
+}
+
 impl fmt::Display for ElfError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            ElfError::BadMagic => write!(f, "Invalid ELF magic number"),
-            ElfError::Malformed => write!(f, "Malformed ELF header"),
-            ElfError::IoError(e) => write!(f, "I/O error: {e}"),
+            Self::BadMagic => write!(f, "Invalid ELF magic number"),
+            Self::Malformed => write!(f, "Malformed ELF header"),
+            Self::IoError(e) => write!(f, "I/O error: {e}"),
         }
     }
 }
 
 impl From<io::Error> for ElfError {
     fn from(err: io::Error) -> Self {
-        ElfError::IoError(err)
+        Self::IoError(err)
     }
 }
 
 impl From<ElfError> for Errno {
     fn from(err: ElfError) -> Self {
         match err {
-            ElfError::BadMagic => Errno::EINVAL,
-            ElfError::Malformed => Errno::EACCES,
-            ElfError::IoError(e) => Errno::from_raw(e.raw_os_error().unwrap_or(Errno::EIO as i32)),
+            ElfError::BadMagic => Self::EINVAL,
+            ElfError::Malformed => Self::EACCES,
+            ElfError::IoError(e) => Self::from_raw(e.raw_os_error().unwrap_or(Self::EIO as i32)),
         }
     }
 }
 
 impl From<Errno> for ElfError {
     fn from(errno: Errno) -> Self {
-        ElfError::IoError(io::Error::from_raw_os_error(errno as i32))
+        Self::IoError(io::Error::from_raw_os_error(errno as i32))
     }
 }
 
 impl fmt::Display for ExecutableFile {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            ExecutableFile::Elf {
+            Self::Elf {
                 elf_type,
                 file_type,
                 linking_type,
@@ -222,7 +242,7 @@ impl fmt::Display for ExecutableFile {
                     write!(f, "ELF:{file_type}{elf_type}{pie}{xs}")
                 }
             }
-            ExecutableFile::Script => write!(f, "SCRIPT"),
+            Self::Script => write!(f, "SCRIPT"),
         }
     }
 }
@@ -230,8 +250,8 @@ impl fmt::Display for ExecutableFile {
 impl fmt::Display for ElfType {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            ElfType::Elf32 => write!(f, "32"),
-            ElfType::Elf64 => write!(f, "64"),
+            Self::Elf32 => write!(f, "32"),
+            Self::Elf64 => write!(f, "64"),
         }
     }
 }
@@ -239,13 +259,13 @@ impl fmt::Display for ElfType {
 impl fmt::Display for ElfFileType {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            ElfFileType::None => write!(f, "none"),
-            ElfFileType::Executable => write!(f, "exe"),
-            ElfFileType::Library => write!(f, "lib"),
-            ElfFileType::Relocatable => write!(f, "rel"),
-            ElfFileType::Core => write!(f, "core"),
-            ElfFileType::ProcessorSpecific => write!(f, "proc"),
-            ElfFileType::Unknown => write!(f, "reserved"),
+            Self::None => write!(f, "none"),
+            Self::Executable => write!(f, "exe"),
+            Self::Library => write!(f, "lib"),
+            Self::Relocatable => write!(f, "rel"),
+            Self::Core => write!(f, "core"),
+            Self::ProcessorSpecific => write!(f, "proc"),
+            Self::Unknown => write!(f, "reserved"),
         }
     }
 }
@@ -253,8 +273,8 @@ impl fmt::Display for ElfFileType {
 impl fmt::Display for LinkingType {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            LinkingType::Static => write!(f, "static"),
-            LinkingType::Dynamic => write!(f, "dynamic"),
+            Self::Static => write!(f, "static"),
+            Self::Dynamic => write!(f, "dynamic"),
         }
     }
 }
@@ -294,22 +314,19 @@ impl ExecutableFile {
         let mut magic = [0u8; 4];
         let mut nread = 0;
         while nread < 4 {
-            #[expect(clippy::arithmetic_side_effects)]
             match reader.read(&mut magic[nread..]) {
-                Ok(0) => break,
-                Ok(n) => nread += n,
+                Ok(0) => {
+                    // Short read, not a valid ELF file.
+                    return Ok(false);
+                }
+                Ok(n) => nread = nread.checked_add(n).ok_or(Errno::EOVERFLOW)?,
                 Err(ref e) if e.kind() == io::ErrorKind::Interrupted => continue,
                 Err(e) => return Err(e.into()),
             }
         }
 
-        if nread < 4 {
-            // Short read, not a valid ELF file.
-            Ok(false)
-        } else {
-            // Check the magic \x7FELF bytes.
-            Ok(magic == ELF_MAGIC)
-        }
+        // Check the magic \x7FELF bytes.
+        Ok(is_equal(&magic, ELF_MAGIC))
     }
 
     /// Checks if the file at the given path is a valid ELF file.
@@ -325,22 +342,19 @@ impl ExecutableFile {
         let mut ident = [0u8; 16]; // EI_NIDENT is 16 bytes.
         let mut nread = 0;
         while nread < 16 {
-            #[expect(clippy::arithmetic_side_effects)]
             match reader.read(&mut ident[nread..]) {
-                Ok(0) => break,
-                Ok(n) => nread += n,
+                Ok(0) => {
+                    // Short read, not a valid ELF file.
+                    return Ok(false);
+                }
+                Ok(n) => nread = nread.checked_add(n).ok_or(Errno::EOVERFLOW)?,
                 Err(ref e) if e.kind() == io::ErrorKind::Interrupted => continue,
                 Err(e) => return Err(e.into()),
             }
         }
 
-        if nread < 16 {
-            // Short read, not a valid ELF file.
-            return Ok(false);
-        }
-
         // Check the magic \x7FELF bytes.
-        if &ident[0..4] != ELF_MAGIC {
+        if !is_equal(&ident[0..4], ELF_MAGIC) {
             return Ok(false);
         }
 
@@ -350,12 +364,12 @@ impl ExecutableFile {
         }
 
         // Check whether the endianness is valid.
-        if ident[EI_DATA] != ELFDATA2LSB && ident[EI_DATA] != ELFDATA2MSB {
+        if !matches!(ident[EI_DATA], ELFDATA2LSB | ELFDATA2MSB) {
             return Ok(false);
         }
 
         // Check the ELF class.
-        if ident[EI_CLASS] != ELFCLASS32 && ident[EI_CLASS] != ELFCLASS64 {
+        if !matches!(ident[EI_CLASS], ELFCLASS64 | ELFCLASS32) {
             return Ok(false);
         }
 
@@ -401,10 +415,9 @@ impl ExecutableFile {
         let mut header = [0u8; 64];
         let mut bytes_read = 0;
         while bytes_read < header.len() {
-            #[expect(clippy::arithmetic_side_effects)]
             match reader.read(&mut header[bytes_read..]) {
                 Ok(0) => break,
-                Ok(n) => bytes_read += n,
+                Ok(n) => bytes_read = bytes_read.checked_add(n).ok_or(Errno::EOVERFLOW)?,
                 Err(ref e) if e.kind() == ErrorKind::Interrupted => continue,
                 Err(e) => return Err(ElfError::IoError(e)),
             }
@@ -421,25 +434,25 @@ impl ExecutableFile {
                 }
             }
             2..=3 => {
-                if header.starts_with(b"#!") {
+                if is_prefix(&header, b"#!") {
                     return Ok(ExecutableFile::Script);
                 } else {
                     return Err(ElfError::BadMagic);
                 }
             }
             4..=63 => {
-                if header.starts_with(b"#!") {
+                if is_prefix(&header, b"#!") {
                     return Ok(ExecutableFile::Script);
-                } else if bytes_read >= 4 && &header[0..4] != ELF_MAGIC {
+                } else if !is_equal(&header[0..4], ELF_MAGIC) {
                     return Err(ElfError::BadMagic);
                 } else {
                     return Err(ElfError::Malformed);
                 }
             }
             _ => {
-                if header.starts_with(b"#!") {
+                if is_prefix(&header, b"#!") {
                     return Ok(ExecutableFile::Script);
-                } else if &header[0..4] != ELF_MAGIC {
+                } else if !is_equal(&header[0..4], ELF_MAGIC) {
                     return Err(ElfError::BadMagic);
                 }
             }
@@ -459,21 +472,14 @@ impl ExecutableFile {
             _ => return Err(ElfError::Malformed),
         };
 
-        // Determine ELF file type (Executable, Library, Relocatable, Core, Processor-specific, Unknown)
-        let e_type = if is_big_endian {
-            read_u16_be(header.get(16..18).ok_or(ElfError::Malformed)?)?
+        // Determine ELF file type:
+        // Executable, Library, Relocatable, Core, Processor-specific, Unknown
+        let mut file_type = if is_big_endian {
+            read_u16_be(header.get(16..18).ok_or(ElfError::Malformed)?)
         } else {
-            read_u16_le(header.get(16..18).ok_or(ElfError::Malformed)?)?
-        };
-        let mut file_type = match e_type {
-            ET_NONE => ElfFileType::None,
-            ET_EXEC => ElfFileType::Executable,
-            ET_DYN => ElfFileType::Library,
-            ET_REL => ElfFileType::Relocatable,
-            ET_CORE => ElfFileType::Core,
-            ET_LOPROC..=ET_HIPROC => ElfFileType::ProcessorSpecific,
-            _ => ElfFileType::Unknown,
-        };
+            read_u16_le(header.get(16..18).ok_or(ElfError::Malformed)?)
+        }
+        .map(ElfFileType::from)?;
 
         // Static or Dynamic linking.
         let mut dynamic = false;
@@ -491,22 +497,27 @@ impl ExecutableFile {
         if check_linking && matches!(file_type, ElfFileType::Executable | ElfFileType::Library) {
             // Read program headers
             let (phoff_offset, phnum_offset, phentsize_offset) = if elf_type == ElfType::Elf64 {
-                (32, 56, 54)
+                (32usize, 56usize, 54usize)
             } else {
-                (28, 44, 42)
+                (28usize, 44usize, 42usize)
             };
-            #[expect(clippy::arithmetic_side_effects)]
             let phoff = if elf_type == ElfType::Elf64 {
                 if is_big_endian {
                     read_u64_be(
                         header
-                            .get(phoff_offset..phoff_offset + 8)
+                            .get(
+                                phoff_offset
+                                    ..phoff_offset.checked_add(8).ok_or(Errno::EOVERFLOW)?,
+                            )
                             .ok_or(ElfError::Malformed)?,
                     )?
                 } else {
                     read_u64_le(
                         header
-                            .get(phoff_offset..phoff_offset + 8)
+                            .get(
+                                phoff_offset
+                                    ..phoff_offset.checked_add(8).ok_or(Errno::EOVERFLOW)?,
+                            )
                             .ok_or(ElfError::Malformed)?,
                     )?
                 }
@@ -514,43 +525,53 @@ impl ExecutableFile {
                 if is_big_endian {
                     read_u32_be(
                         header
-                            .get(phoff_offset..phoff_offset + 4)
+                            .get(
+                                phoff_offset
+                                    ..phoff_offset.checked_add(4).ok_or(Errno::EOVERFLOW)?,
+                            )
                             .ok_or(ElfError::Malformed)?,
                     )?
                 } else {
                     read_u32_le(
                         header
-                            .get(phoff_offset..phoff_offset + 4)
+                            .get(
+                                phoff_offset
+                                    ..phoff_offset.checked_add(4).ok_or(Errno::EOVERFLOW)?,
+                            )
                             .ok_or(ElfError::Malformed)?,
                     )?
                 }
                 .into()
             };
-            #[expect(clippy::arithmetic_side_effects)]
             let phnum = if is_big_endian {
                 read_u16_be(
                     header
-                        .get(phnum_offset..phnum_offset + 2)
+                        .get(phnum_offset..phnum_offset.checked_add(2).ok_or(Errno::EOVERFLOW)?)
                         .ok_or(ElfError::Malformed)?,
                 )?
             } else {
                 read_u16_le(
                     header
-                        .get(phnum_offset..phnum_offset + 2)
+                        .get(phnum_offset..phnum_offset.checked_add(2).ok_or(Errno::EOVERFLOW)?)
                         .ok_or(ElfError::Malformed)?,
                 )?
             } as usize;
-            #[expect(clippy::arithmetic_side_effects)]
             let phentsize = if is_big_endian {
                 read_u16_be(
                     header
-                        .get(phentsize_offset..phentsize_offset + 2)
+                        .get(
+                            phentsize_offset
+                                ..phentsize_offset.checked_add(2).ok_or(Errno::EOVERFLOW)?,
+                        )
                         .ok_or(ElfError::Malformed)?,
                 )?
             } else {
                 read_u16_le(
                     header
-                        .get(phentsize_offset..phentsize_offset + 2)
+                        .get(
+                            phentsize_offset
+                                ..phentsize_offset.checked_add(2).ok_or(Errno::EOVERFLOW)?,
+                        )
                         .ok_or(ElfError::Malformed)?,
                 )?
             } as usize;
@@ -566,9 +587,9 @@ impl ExecutableFile {
             }
 
             // Allocate memory for program headers safely.
-            let total_size = phnum.checked_mul(phentsize).ok_or(ElfError::Malformed)?;
+            let total_size = phnum.checked_mul(phentsize).ok_or(Errno::EOVERFLOW)?;
             let mut phdrs = Vec::new();
-            phdrs.try_reserve(total_size).or(Err(ElfError::Malformed))?;
+            phdrs.try_reserve(total_size).or(Err(Errno::ENOMEM))?;
             phdrs.resize(total_size, 0);
             reader.seek(SeekFrom::Start(phoff))?;
             reader.read_exact(&mut phdrs)?;
@@ -579,8 +600,8 @@ impl ExecutableFile {
             // PT_GNU_STACK: Last header is used, rest are ignored.
             let mut seen_interp = false;
             for i in 0..phnum {
-                let offset = i.checked_mul(phentsize).ok_or(ElfError::Malformed)?;
-                let end = offset.checked_add(4).ok_or(ElfError::Malformed)?;
+                let offset = i.checked_mul(phentsize).ok_or(Errno::EOVERFLOW)?;
+                let end = offset.checked_add(4).ok_or(Errno::EOVERFLOW)?;
                 if end > phdrs.len() || offset >= phdrs.len() {
                     // If the offset plus the size of the field (4
                     // bytes) exceeds the length of the program header
@@ -611,14 +632,14 @@ impl ExecutableFile {
                         // header. In 32-bit ELF, p_flags is at offset
                         // 24 from the start of the program header.
                         let flags_offset = if elf_type == ElfType::Elf64 {
-                            offset.checked_add(4).ok_or(ElfError::Malformed)?
+                            offset.checked_add(4).ok_or(Errno::EOVERFLOW)?
                         } else {
-                            offset.checked_add(24).ok_or(ElfError::Malformed)?
+                            offset.checked_add(24).ok_or(Errno::EOVERFLOW)?
                         };
 
                         // `p_flags` is always 4 bytes in both 32-bit
                         // and 64-bit ELF headers.
-                        let flags_end = flags_offset.checked_add(4).ok_or(ElfError::Malformed)?;
+                        let flags_end = flags_offset.checked_add(4).ok_or(Errno::EOVERFLOW)?;
 
                         // Check sanity of offsets.
                         if flags_end > phdrs.len() || flags_offset >= phdrs.len() {
@@ -652,10 +673,8 @@ impl ExecutableFile {
             }
         }
 
-        // Linking type should be None for libraries
-        let linking_type = if file_type == ElfFileType::Library {
-            None
-        } else if file_type == ElfFileType::Executable {
+        // Linking type should be None for non-executables.
+        let linking_type = if file_type == ElfFileType::Executable {
             if dynamic {
                 Some(LinkingType::Dynamic)
             } else {
@@ -688,7 +707,7 @@ fn is_pie(
     };
 
     for i in (0..dynamic_size).step_by(entry_size) {
-        let j = i.checked_add(entry_size / 2).ok_or(ElfError::Malformed)?;
+        let j = i.checked_add(entry_size / 2).ok_or(Errno::EOVERFLOW)?;
         if j > dynamic_size || i >= dynamic_size {
             // See the comment in parse().
             break;
@@ -708,7 +727,7 @@ fn is_pie(
             }
         };
         if d_tag == DT_FLAGS_1 {
-            let k = i.checked_add(entry_size).ok_or(ElfError::Malformed)?;
+            let k = i.checked_add(entry_size).ok_or(Errno::EOVERFLOW)?;
             if k > dynamic_size || j >= dynamic_size {
                 // See the comment in parse().
                 break;
@@ -745,8 +764,8 @@ fn read_dynamic_section<R: Read + Seek>(
     phentsize: usize,
 ) -> Result<Option<(Vec<u8>, usize)>, ElfError> {
     for i in 0..phnum {
-        let offset = i.checked_mul(phentsize).ok_or(ElfError::Malformed)?;
-        let end = offset.checked_add(4).ok_or(ElfError::Malformed)?;
+        let offset = i.checked_mul(phentsize).ok_or(Errno::EOVERFLOW)?;
+        let end = offset.checked_add(4).ok_or(Errno::EOVERFLOW)?;
         if end > phdrs.len() || offset >= phdrs.len() {
             // See the comment in parse().
             break;
@@ -758,8 +777,8 @@ fn read_dynamic_section<R: Read + Seek>(
         };
         if p_type == PT_DYNAMIC {
             let p_offset = if elf_type == ElfType::Elf64 {
-                let offset_dyn_min = offset.checked_add(8).ok_or(ElfError::Malformed)?;
-                let offset_dyn_max = offset.checked_add(16).ok_or(ElfError::Malformed)?;
+                let offset_dyn_min = offset.checked_add(8).ok_or(Errno::EOVERFLOW)?;
+                let offset_dyn_max = offset.checked_add(16).ok_or(Errno::EOVERFLOW)?;
                 if offset_dyn_max > phdrs.len() || offset_dyn_min >= phdrs.len() {
                     // See the comment in parse().
                     break;
@@ -770,8 +789,8 @@ fn read_dynamic_section<R: Read + Seek>(
                     read_u64_le(&phdrs[offset_dyn_min..offset_dyn_max])?
                 }
             } else {
-                let offset_dyn_min = offset.checked_add(4).ok_or(ElfError::Malformed)?;
-                let offset_dyn_max = offset.checked_add(8).ok_or(ElfError::Malformed)?;
+                let offset_dyn_min = offset.checked_add(4).ok_or(Errno::EOVERFLOW)?;
+                let offset_dyn_max = offset.checked_add(8).ok_or(Errno::EOVERFLOW)?;
                 if offset_dyn_max > phdrs.len() || offset_dyn_min >= phdrs.len() {
                     // See the comment in parse().
                     break;
@@ -783,8 +802,8 @@ fn read_dynamic_section<R: Read + Seek>(
                 }
             };
             let p_filesz = if elf_type == ElfType::Elf64 {
-                let offset_filesz_min = offset.checked_add(32).ok_or(ElfError::Malformed)?;
-                let offset_filesz_max = offset.checked_add(40).ok_or(ElfError::Malformed)?;
+                let offset_filesz_min = offset.checked_add(32).ok_or(Errno::EOVERFLOW)?;
+                let offset_filesz_max = offset.checked_add(40).ok_or(Errno::EOVERFLOW)?;
                 if offset_filesz_max > phdrs.len() || offset_filesz_min >= phdrs.len() {
                     // See the comment in parse().
                     break;
@@ -795,8 +814,8 @@ fn read_dynamic_section<R: Read + Seek>(
                     read_u64_le(&phdrs[offset_filesz_min..offset_filesz_max])?
                 }
             } else {
-                let offset_filesz_min = offset.checked_add(16).ok_or(ElfError::Malformed)?;
-                let offset_filesz_max = offset.checked_add(20).ok_or(ElfError::Malformed)?;
+                let offset_filesz_min = offset.checked_add(16).ok_or(Errno::EOVERFLOW)?;
+                let offset_filesz_max = offset.checked_add(20).ok_or(Errno::EOVERFLOW)?;
                 if offset_filesz_max > phdrs.len() || offset_filesz_min >= phdrs.len() {
                     // See the comment in parse().
                     break;
@@ -823,7 +842,7 @@ fn read_dynamic_section<R: Read + Seek>(
             let p_filesz = usize::try_from(p_filesz).or(Err(ElfError::Malformed))?;
             dynamic_section
                 .try_reserve(p_filesz)
-                .or(Err(ElfError::Malformed))?;
+                .or(Err(Errno::ENOMEM))?;
             dynamic_section.resize(p_filesz, 0);
             reader.read_exact(&mut dynamic_section)?;
 
