@@ -16,8 +16,8 @@ use nix::{errno::Errno, fcntl::AtFlags, unistd::AccessFlags, NixPath};
 
 use crate::{
     fs::{fdaccess, FsFlags},
-    hook::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{syscall_path_handler, to_atflags},
+    req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_access(request: UNotifyEventRequest) -> ScmpNotifResp {
