@@ -22,10 +22,10 @@ use crate::{
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err2no, error,
     fs::{is_executable, safe_open_msym, FileType, FsFlags, MaybeFd, AT_EXECVE_CHECK},
-    hook::{RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{sandbox_path, to_atflags},
     path::XPathBuf,
     ptrace::ptrace_syscall_info,
+    req::{RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
     sandbox::{Action, Capability, IntegrityError, SandboxGuard},
     warn,
 };
@@ -538,7 +538,7 @@ fn syscall_exec_handler(
         let restrict_memfd = !sandbox.flags.allow_unsafe_memfd();
 
         // Read remote path.
-        let (path, _) = request.read_path(&sandbox, arg, false)?;
+        let (path, _, _) = request.read_path(&sandbox, arg, false)?;
 
         // Call sandbox access checker if Exec sandboxing is on.
         if sandbox.enabled(Capability::CAP_EXEC) {
