@@ -81,6 +81,9 @@ pub(crate) mod open;
 /// prctl(2) handler
 pub(crate) mod prctl;
 
+/// readlink(2) and readlinkat(2) handlers
+pub(crate) mod readlink;
+
 /// rename(2), renameat(2) and renameat2(2) handlers
 pub(crate) mod rename;
 
@@ -142,10 +145,10 @@ use nix::{
 
 use crate::{
     fs::{fd_status_flags, to_valid_fd, CanonicalPath, FileInfo, FileType},
-    hook::{PathArgs, SysArg, UNotifyEventRequest},
     ioctl::{ioctl_names_get, Ioctl},
     notice,
     path::{XPath, XPathBuf},
+    req::{PathArgs, SysArg, UNotifyEventRequest},
     sandbox::{Action, Capability, Sandbox, SandboxGuard},
     warn,
 };
@@ -401,11 +404,9 @@ where
         let crypt = sandbox.enabled(Capability::CAP_CRYPT);
         let hide = !is_fd && sandbox.enabled(Capability::CAP_STAT);
 
-        // EXCEPTION: We do want to return success t o _access_(2) calls
+        // EXCEPTION: We do want to return success to _access_(2) calls
         // to magic paths in case the sandbox lock allows it.
-        let mut magic = !is_fd
-            && !sandbox.locked_for(req.pid())
-            && memmem::find(syscall_name.as_bytes(), b"access").is_some();
+        let mut magic = !is_fd && memmem::find(syscall_name.as_bytes(), b"access").is_some();
 
         let mut paths: [Option<CanonicalPath>; 2] = [None, None];
         for (idx, arg) in path_argv.iter().enumerate() {
@@ -413,7 +414,7 @@ where
             // such as fchmod, fchown, falllocate, ftruncate,
             // fgetxattr, fsetxattr safely and efficiently.
             if arg.path.is_some() {
-                let (path, is_magic) = request.read_path(&sandbox, *arg, magic)?;
+                let (path, is_magic, _) = request.read_path(&sandbox, *arg, magic)?;
                 magic = is_magic;
 
                 if sandbox.is_chroot() {
@@ -480,6 +481,11 @@ where
             }
         }
 
+        if magic && sandbox.locked_for(req.pid()) {
+            // Sandbox is locked, access denied.
+            return Err(Errno::ENOENT);
+        }
+
         if !magic {
             // Call sandbox access checker, skip magic paths.
             match (&paths[0], &paths[1]) {
