@@ -13,25 +13,25 @@ use std::{
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    sys::socket::{sendmsg, ControlMessage, MsgFlags, SockaddrStorage, UnixCredentials},
+    sys::socket::{sendmsg, ControlMessage, SockaddrStorage, UnixCredentials},
     unistd::Pid,
 };
 use zeroize::Zeroizing;
 
 use crate::{
     compat::{
-        addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, PF_ALG, PF_INET, PF_INET6,
-        PF_NETLINK, PF_UNIX,
+        addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, MsgFlags, PF_ALG, PF_INET,
+        PF_INET6, PF_NETLINK, PF_UNIX,
     },
     confine::scmp_arch_bits,
-    fs::{file_type, get_nonblock, has_recv_timeout, is_same_vm, FileType},
-    hook::UNotifyEventRequest,
+    fs::{get_nonblock, has_recv_timeout, is_same_vm},
     kernel::{
         net::{canon_addr, get_addr, sandbox_addr, to_msgflags},
         sandbox_path,
     },
     path::XPath,
-    sandbox::Capability,
+    req::UNotifyEventRequest,
+    sandbox::{Capability, Flags},
 };
 
 #[expect(clippy::cognitive_complexity)]
@@ -39,15 +39,13 @@ pub(crate) fn handle_sendmsg(
     fd: OwnedFd,
     request: &UNotifyEventRequest,
     args: &[u64; 6],
-    allow_unsupp_socket: bool,
-    restrict_oob: bool,
-    restrict_mkbdev: bool,
+    flags: Flags,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Reject undefined/invalid flags.
-    let flags = to_msgflags(args[2])?;
+    let msgflags = to_msgflags(args[2])?;
 
     // SAFETY: Reject MSG_OOB as necessary.
-    if restrict_oob && flags.contains(MsgFlags::MSG_OOB) {
+    if !flags.allow_unsafe_oob() && msgflags.contains(MsgFlags::MSG_OOB) {
         // Signal no support to let the sandbox process
         // handle the error gracefully. This is consistent
         // with the Linux kernel.
@@ -90,6 +88,7 @@ pub(crate) fn handle_sendmsg(
 
     // Step 1: Handle the address.
     let sandbox = request.get_sandbox();
+    let verbose = sandbox.verbose;
     let addr = if !addr_remote.is_null() && addr_len > 0 {
         let addr = get_addr(request, addr_remote as u64, addr_len)?;
         let (addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
@@ -108,7 +107,7 @@ pub(crate) fn handle_sendmsg(
             PF_ALG | PF_NETLINK => {
                 // SAFETY: We do not check AF_ALG or AF_NETLINK for access.
             }
-            _ if allow_unsupp_socket => {
+            _ if flags.allow_unsupp_socket() => {
                 // SAFETY: We do not check unsupported sockets for access.
             }
             _ => return Err(Errno::EAFNOSUPPORT),
@@ -181,22 +180,9 @@ pub(crate) fn handle_sendmsg(
         }
 
         for fds in control_fds {
-            // SAFETY: Deny sending file descriptors referring to
-            // 1. Block devices unless trace/allow_unsafe_mkbdev:1 is set.
-            // 2. Directories
-            // 3. Symbolic links
-            //
-            // Note, we do allow files of unknown type such as epoll
-            // fds and event fds as some programs such as pipewire
-            // depend on this. See test-pw-filter test of pipewire
-            // for more information about this.
+            // SAFETY: Check safe file types for SCM_RIGHTS.
             for fd in fds {
-                // TODO: Log this deny!
-                match file_type(fd, None, false)? {
-                    FileType::Dir | FileType::Lnk => return Err(Errno::EACCES),
-                    FileType::Blk if restrict_mkbdev => return Err(Errno::EACCES),
-                    _ => {}
-                }
+                request.check_scm_rights(&fd, flags, 0x10, verbose)?;
             }
 
             // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -332,7 +318,7 @@ pub(crate) fn handle_sendmsg(
     }
 
     // SAFETY: Record blocking call so it can get invalidated.
-    let is_blocking = if !flags.contains(MsgFlags::MSG_DONTWAIT) && !get_nonblock(&fd)? {
+    let is_blocking = if !msgflags.contains(MsgFlags::MSG_DONTWAIT) && !get_nonblock(&fd)? {
         let req = request.scmpreq;
         let ignore_restart = has_recv_timeout(&fd)?;
 
@@ -350,12 +336,18 @@ pub(crate) fn handle_sendmsg(
             fd.as_raw_fd(),
             &io_slices,
             &control_messages,
-            flags,
+            msgflags.into(),
             Some(&addr),
         )
     } else {
         // Connection-mode socket.
-        sendmsg::<SockaddrStorage>(fd.as_raw_fd(), &io_slices, &control_messages, flags, None)
+        sendmsg::<SockaddrStorage>(
+            fd.as_raw_fd(),
+            &io_slices,
+            &control_messages,
+            msgflags.into(),
+            None,
+        )
     };
 
     // Remove invalidation record unless interrupted.
@@ -369,7 +361,7 @@ pub(crate) fn handle_sendmsg(
     #[expect(clippy::cast_possible_wrap)]
     Ok(match result {
         Ok(n) => request.return_syscall(n as i64),
-        Err(Errno::EPIPE) if !flags.contains(MsgFlags::MSG_NOSIGNAL) => {
+        Err(Errno::EPIPE) if !msgflags.contains(MsgFlags::MSG_NOSIGNAL) => {
             request.pidfd_kill(libc::SIGPIPE)?;
             request.fail_syscall(Errno::EPIPE)
         }
@@ -382,15 +374,13 @@ pub(crate) fn handle_sendmmsg(
     fd: OwnedFd,
     request: &UNotifyEventRequest,
     args: &[u64; 6],
-    allow_unsupp_socket: bool,
-    restrict_oob: bool,
-    restrict_mkbdev: bool,
+    flags: Flags,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Reject undefined/invalid flags.
     let msgflags = to_msgflags(args[3])?;
 
     // SAFETY: Reject MSG_OOB as necessary.
-    if restrict_oob && msgflags.contains(MsgFlags::MSG_OOB) {
+    if !flags.allow_unsafe_oob() && msgflags.contains(MsgFlags::MSG_OOB) {
         // Signal no support to let the sandbox process
         // handle the error gracefully. This is consistent
         // with the Linux kernel.
@@ -456,6 +446,7 @@ pub(crate) fn handle_sendmmsg(
         let addr_remote = msg.msg_name;
         let addr_len = msg.msg_namelen;
         let sandbox = request.get_sandbox();
+        let verbose = sandbox.verbose;
         let addr_root = if !addr_remote.is_null() && addr_len > 0 {
             let addr = get_addr(request, addr_remote as u64, addr_len)?;
             let (addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
@@ -474,7 +465,7 @@ pub(crate) fn handle_sendmmsg(
                 PF_ALG | PF_NETLINK => {
                     // SAFETY: We do not check AF_ALG and AF_NETLINK for access.
                 }
-                _ if allow_unsupp_socket => {
+                _ if flags.allow_unsupp_socket() => {
                     // SAFETY: We do not check unsupported sockets for access.
                 }
                 _ => return Err(Errno::EAFNOSUPPORT),
@@ -531,7 +522,7 @@ pub(crate) fn handle_sendmmsg(
                             &sandbox,
                             addr,
                             root,
-                            0x10,
+                            0x14,
                             Capability::CAP_NET_SENDFD,
                         )?;
                     } else {
@@ -551,22 +542,9 @@ pub(crate) fn handle_sendmmsg(
                 }
 
                 for fds in control_fds {
-                    // SAFETY: Deny sending file descriptors referring to
-                    // 1. Block devices unless trace/allow_unsafe_mkbdev:1 is set.
-                    // 2. Directories
-                    // 3. Symbolic links
-                    //
-                    // Note, we do allow files of unknown type such as epoll
-                    // fds and event fds as some programs such as pipewire
-                    // depend on this. See test-pw-filter test of pipewire
-                    // for more information about this.
+                    // SAFETY: Check safe file types for SCM_RIGHTS.
                     for fd in fds {
-                        // TODO: Log this deny!
-                        match file_type(fd, None, false)? {
-                            FileType::Dir | FileType::Lnk => return Err(Errno::EACCES),
-                            FileType::Blk if restrict_mkbdev => return Err(Errno::EACCES),
-                            _ => {}
-                        }
+                        request.check_scm_rights(&fd, flags, 0x14, verbose)?;
                     }
 
                     // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -701,7 +679,7 @@ pub(crate) fn handle_sendmmsg(
                     fd.as_raw_fd(),
                     &io_slices,
                     &control_messages,
-                    msgflags,
+                    msgflags.into(),
                     Some(&addr),
                 )
             } else {
@@ -710,7 +688,7 @@ pub(crate) fn handle_sendmmsg(
                     fd.as_raw_fd(),
                     &io_slices,
                     &control_messages,
-                    msgflags,
+                    msgflags.into(),
                     None,
                 )
             };
@@ -1073,8 +1051,7 @@ fn parse_control_messages(
                 }
                 // SAFETY: Exact u8-sized payload validated; bounds and alignment guaranteed
                 // by prior checks and cmsg parsing semantics. Single u8 read is safe.
-                #[expect(clippy::cast_ptr_alignment)]
-                let val = unsafe { *(data.as_ptr() as *const u8) };
+                let val = unsafe { *data.as_ptr() };
                 control_ipv4_toses.push(val);
             }
             (libc::IPPROTO_IPV6, libc::IPV6_TCLASS) => {
