@@ -14,8 +14,8 @@ use nix::{errno::Errno, unistd::UnlinkatFlags};
 use crate::{
     cookie::safe_unlinkat,
     fs::FsFlags,
-    hook::{PathArgs, SysArg, UNotifyEventRequest},
     kernel::syscall_path_handler,
+    req::{PathArgs, SysArg, UNotifyEventRequest},
 };
 
 pub(crate) fn sys_rmdir(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -27,25 +27,11 @@ pub(crate) fn sys_rmdir(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::MUST_PATH | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "rmdir",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            drop(sandbox); // release the read-lock.
+    syscall_path_handler(request, "rmdir", argv, |path_args, request, sandbox| {
+        drop(sandbox); // release the read-lock.
 
-            // SAFETY: SysArg has one element.
-            #[allow(clippy::disallowed_methods)]
-            let path = path_args.0.as_ref().unwrap();
-
-            safe_unlinkat(
-                path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?,
-                path.base,
-                UnlinkatFlags::RemoveDir,
-            )
-            .map(|_| request.return_syscall(0))
-        },
-    )
+        syscall_unlink_handler(request, path_args, UnlinkatFlags::RemoveDir)
+    })
 }
 
 pub(crate) fn sys_unlink(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -57,25 +43,11 @@ pub(crate) fn sys_unlink(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "unlink",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            drop(sandbox); // release the read-lock.
+    syscall_path_handler(request, "unlink", argv, |path_args, request, sandbox| {
+        drop(sandbox); // release the read-lock.
 
-            // SAFETY: SysArg has one element.
-            #[allow(clippy::disallowed_methods)]
-            let path = path_args.0.as_ref().unwrap();
-
-            safe_unlinkat(
-                path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?,
-                path.base,
-                UnlinkatFlags::NoRemoveDir,
-            )
-            .map(|_| request.return_syscall(0))
-        },
-    )
+        syscall_unlink_handler(request, path_args, UnlinkatFlags::NoRemoveDir)
+    })
 }
 
 pub(crate) fn sys_unlinkat(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -105,23 +77,23 @@ pub(crate) fn sys_unlinkat(request: UNotifyEventRequest) -> ScmpNotifResp {
         fsflags: FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH | FsFlags::WANT_BASE,
         ..Default::default()
     }];
-    syscall_path_handler(
-        request,
-        "unlinkat",
-        argv,
-        |path_args: PathArgs, request, sandbox| {
-            drop(sandbox); // release the read-lock.
+    syscall_path_handler(request, "unlinkat", argv, |path_args, request, sandbox| {
+        drop(sandbox); // release the read-lock.
 
-            // SAFETY: SysArg has one element.
-            #[allow(clippy::disallowed_methods)]
-            let path = path_args.0.as_ref().unwrap();
+        syscall_unlink_handler(request, path_args, flags)
+    })
+}
 
-            safe_unlinkat(
-                path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?,
-                path.base,
-                flags,
-            )
-            .map(|_| request.return_syscall(0))
-        },
-    )
+/// A helper function to handle rmdir and unlink{,at} syscalls.
+fn syscall_unlink_handler(
+    request: &UNotifyEventRequest,
+    args: PathArgs,
+    flags: UnlinkatFlags,
+) -> Result<ScmpNotifResp, Errno> {
+    // SAFETY: SysArg has one element.
+    #[expect(clippy::disallowed_methods)]
+    let path = args.0.as_ref().unwrap();
+    let dir = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
+
+    safe_unlinkat(dir, path.base, flags).map(|_| request.return_syscall(0))
 }
