@@ -29,8 +29,10 @@ use serde_json::{Map, Value};
 use crate::{
     config::*,
     err::SydResult,
-    fs::{is_active_fd, lock_fd, retry_on_eintr, unlock_fd},
+    fs::is_active_fd,
+    ofd::{lock_fd, unlock_fd},
     proc::{proc_cmdline, proc_comm, proc_cwd, proc_tty},
+    retry::retry_on_eintr,
     syslog::LogLevel,
 };
 
@@ -538,6 +540,9 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         msg.insert("tip".to_string(), tip);
     }
 
+    // Remove NULL elements to save space.
+    msg.retain(|_, value| !value.is_null());
+
     // Convert to JSON line.
     let msg_data = serde_json::to_string(&msg).unwrap_or_else(|e| {
         let e = serde_json::to_string(&format!("{e:?}")).unwrap_or("?".to_string());
@@ -715,6 +720,9 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         msg.insert("tip".to_string(), tip);
     }
 
+    // Remove NULL elements to save space.
+    msg.retain(|_, value| !value.is_null());
+
     // Convert to JSON line
     let msg_data = serde_json::to_string(&msg).unwrap_or_else(|e| {
         let e = serde_json::to_string(&format!("{e:?}")).unwrap_or("?".to_string());
@@ -812,6 +820,11 @@ impl Tm {
         self.tm_mday
     }
 
+    /// Get the week day of the date, where Sunday=0.
+    pub fn weekday(&self) -> i32 {
+        self.tm_wday
+    }
+
     /// Get the hour of the date.
     pub fn hour(&self) -> i32 {
         self.tm_hour
