@@ -38,8 +38,9 @@ use nix::{
 
 use crate::{
     config::MAGIC_PREFIX,
-    fs::{retry_on_eintr, tgkill, FileType},
+    fs::{tgkill, FileType},
     log::log_untrusted_buf,
+    retry::retry_on_eintr,
 };
 
 /// Generate a formatted `XPathBuf`.
@@ -1334,19 +1335,13 @@ impl XPathBuf {
         self.0.try_reserve(additional).or(Err(Errno::ENOMEM))
     }
 
-    /// Create a new, empty `XPathBuf`.
-    pub fn empty() -> Self {
-        Self(vec![])
-    }
-
-    /// Construct an empty `XPathBuf` with capacity pre-allocated.
-    pub fn with_capacity(n: usize) -> Self {
-        Self(Vec::with_capacity(n))
-    }
-
-    /// Report capacity of path.
-    pub fn capacity(&self) -> usize {
-        self.0.capacity()
+    /// Resizes the `XPathBuf` in place so that `len` is equal to `new_len`.
+    ///
+    /// If `new_len` is greater than `len`, the `XPathBuf` is extended by the difference,
+    /// with each additional slot filled with value. If `new_len` is less than `len`, the
+    /// `XPathBuf` is simply truncated.
+    pub fn resize(&mut self, new_len: usize, value: u8) {
+        self.0.resize(new_len, value)
     }
 
     /// Creates an owned `XPathBuf` with path adjoined to `self`.
@@ -1432,6 +1427,30 @@ impl XPathBuf {
     pub unsafe fn set_len(&mut self, new_len: usize) {
         self.0.set_len(new_len)
     }
+
+    /// Construct an empty `XPathBuf` with capacity pre-allocated.
+    pub fn with_capacity(n: usize) -> Self {
+        Self(Vec::with_capacity(n))
+    }
+
+    /// Report capacity of path.
+    pub fn capacity(&self) -> usize {
+        self.0.capacity()
+    }
+
+    /// Constructs a new, empty `XPathBuf`.
+    ///
+    /// The internal vector will not allocate until elements are pushed onto it.
+    pub const fn empty() -> Self {
+        Self::new()
+    }
+
+    /// Constructs a new, empty `XPathBuf`.
+    ///
+    /// The internal vector will not allocate until elements are pushed onto it.
+    pub const fn new() -> XPathBuf {
+        Self(Vec::new())
+    }
 }
 
 /// Logs an untrusted Path, escaping it as hex if it contains control
