@@ -44,13 +44,15 @@ use crate::{
     confine::ExportMode,
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
-    fs::{block_signal, closeexcept, retry_on_eintr, retry_on_intr, seccomp_export_pfc},
+    fs::{block_signal, closeexcept, seccomp_export_pfc},
     hash::SydHashMap,
     hook::HandlerMap,
-    info, rwrite, rwriteln,
+    info,
+    retry::{retry_on_eintr, retry_on_intr},
+    rwrite, rwriteln,
     sandbox::{Capability, Flags, Sandbox},
     workers::{
-        aes::{AesMap, AesWorker},
+        aes::{AesLock, AesWorker},
         emu::Worker,
         int::Interrupter,
         ipc::IpcWorker,
@@ -71,7 +73,7 @@ pub(crate) struct ThreadPool {
     pub(crate) cache: Arc<WorkerCache<'static>>,
     sandbox: Arc<RwLock<Sandbox>>,
     handlers: Arc<HandlerMap>,
-    crypt_map: Option<AesMap>,
+    crypt_map: Option<AesLock>,
     unix_map: UnixMap,
     should_exit: Arc<AtomicBool>,
     worker_data: Arc<WorkerData>,
@@ -87,8 +89,7 @@ impl ThreadPool {
     /// `keep_alive` specifies the duration in milliseconds for which to
     /// keep non-core pool worker threads alive while they do not
     /// receive any work.
-    #[allow(clippy::cognitive_complexity)]
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub(crate) fn new(
         seccomp_fd: RawFd,
         flags: Flags,
@@ -97,7 +98,7 @@ impl ThreadPool {
         sandbox: Arc<RwLock<Sandbox>>,
         handlers: Arc<HandlerMap>,
         should_exit: Arc<AtomicBool>,
-        crypt_map: Option<AesMap>,
+        crypt_map: Option<AesLock>,
     ) -> Self {
         Self {
             sandbox,
@@ -115,12 +116,11 @@ impl ThreadPool {
     }
 
     /// Boot the thread pool. This is the main entry point.
-    #[allow(clippy::cognitive_complexity)]
     pub(crate) fn boot(self) -> SydResult<SydJoinHandle<()>> {
         // Export seccomp rules if requested.
         // We have to prepare the filter twice if exporting,
         // as we cannot move it safely between threads...
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         match ExportMode::from_env() {
             Some(ExportMode::BerkeleyPacketFilter) => {
                 // Worker rules
@@ -234,7 +234,7 @@ impl ThreadPool {
     /// ensure a sandbox process cannot DOS Syd by merely exhausting
     /// workers by e.g. opening the read end of a FIFO over and over
     /// again.
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn monitor(self) -> SydResult<SydJoinHandle<()>> {
         thread::Builder::new()
             .name("syd_mon".to_string())
@@ -267,7 +267,7 @@ impl ThreadPool {
 
                 // SAFETY: The monitor thread needs to inherit FDs.
                 // We have to sort the set as the FDs are randomized.
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_sign_loss)]
                 let mut set = vec![
                     ROOT_FD() as libc::c_uint,
                     PROC_FD() as libc::c_uint,
@@ -276,7 +276,7 @@ impl ThreadPool {
                     self.seccomp_fd as libc::c_uint,
                     crate::log::LOG_FD.load(Ordering::Relaxed) as libc::c_uint,
                 ];
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_sign_loss)]
                 if let Some((crypt_fds, crypt_tmp)) = crypt {
                     set.push(crypt_fds.0 as libc::c_uint);
                     set.push(crypt_fds.1 as libc::c_uint);
@@ -502,7 +502,7 @@ impl ThreadPool {
     pub(crate) fn try_spawn_aes(
         &self,
         fdalg: (RawFd, RawFd),
-        files: AesMap,
+        files: AesLock,
         memfd: bool,
         should_exit: Arc<AtomicBool>,
         transit_uids: &[(Uid, Uid)],
@@ -527,8 +527,8 @@ impl ThreadPool {
     /// Try to create a new worker thread as needed.
     /// Returns Ok(Some((SydJoinHandle, bool))) if spawn succeeded, Ok(None) if no spawn was needed.
     /// The boolean in the success case is true if the thread we spawned was a core thread.
-    #[allow(clippy::cognitive_complexity)]
-    #[allow(clippy::type_complexity)]
+    #[expect(clippy::cognitive_complexity)]
+    #[expect(clippy::type_complexity)]
     pub(crate) fn try_spawn(
         &self,
         ctx: Option<&ScmpFilterContext>,
