@@ -52,6 +52,7 @@ use nix::{
         uio::{process_vm_readv, process_vm_writev, RemoteIoVec},
     },
     unistd::{getpid, lseek64, write, Gid, Pid, Uid, Whence},
+    NixPath,
 };
 use procfs_core::{
     net::UnixNetEntries,
@@ -66,7 +67,7 @@ use crate::{
     cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
     elf::ElfType,
     err::{err2no, proc_error_to_errno},
-    fs::{is_open_fd, parse_fd, read_all, retry_on_eintr, safe_open, write_all, AT_BADFD},
+    fs::{is_open_fd, parse_fd, read_all, safe_open, write_all, AT_BADFD},
     hash::{SydHashMap, SydHashSet, SydIndexSet},
     parsers::{
         map_result,
@@ -74,6 +75,7 @@ use crate::{
         read_to_end,
     },
     path::{XPathBuf, PATH_MAX},
+    retry::retry_on_eintr,
     rng::randint,
     sigset::SydSigSet,
     XPath,
@@ -300,7 +302,7 @@ pub fn procmap_query<Fd: AsFd>(
     // SAFETY:
     // `q` is #[repr(C)] and fully initialized;
     // buffers live at least until ioctl(2) returns.
-    #[allow(unsafe_code)]
+    #[expect(unsafe_code)]
     Errno::result(unsafe {
         libc::syscall(
             libc::SYS_ioctl,
@@ -385,7 +387,7 @@ fn proc_errno(err: Errno) -> Errno {
 }
 
 /// Function to safely open a proc(5) file and return an `OwnedFd`.
-pub fn safe_open_proc(path: &XPath) -> Result<OwnedFd, Errno> {
+pub fn safe_open_proc<P: NixPath + ?Sized>(path: &P) -> Result<OwnedFd, Errno> {
     safe_open(PROC_FILE(), path, OFLAG_PROC, RFLAG_PROC)
 }
 
@@ -441,7 +443,7 @@ fn read_usize_from_ne_bytes(bytes: &[u8], ptr_size: usize) -> Result<usize, Errn
 }
 
 fn usize_to_ne_bytes(value: usize, sizeof_ptr: usize) -> Vec<u8> {
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     match sizeof_ptr {
         4 => (value as u32).to_ne_bytes().to_vec(),
         8 => (value as u64).to_ne_bytes().to_vec(),
@@ -664,7 +666,7 @@ pub fn proc_cmdline(pid: Pid) -> Result<XPathBuf, Errno> {
 
     let mut data = data.to_vec();
     // Determine if EOF was reached or if we hit the limit
-    #[allow(clippy::arithmetic_side_effects)]
+    #[expect(clippy::arithmetic_side_effects)]
     if nread <= 1 {
         // Empty cmdline.
         return Ok(XPathBuf::empty());
@@ -758,7 +760,7 @@ pub fn proc_pipemax() -> Result<libc::c_int, Errno> {
 
 /// Open a FD to /proc/sys/fs/pipe-max-size and return an OwnedFd.
 pub fn proc_pipemax_open() -> Result<OwnedFd, Errno> {
-    safe_open_proc(XPath::from_bytes(b"sys/fs/pipe-max-size"))
+    safe_open_proc("sys/fs/pipe-max-size")
 }
 
 /// Retrieve the system pipe max limit from the given Fd.
@@ -774,13 +776,8 @@ pub fn proc_pipemax_read<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
 pub fn proc_fs_file_max() -> Result<u64, Errno> {
     // SAFETY: Informational command,
     // runs before syd::config::proc_init.
-    #[allow(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(
-        AT_BADFD,
-        XPath::from_bytes(b"/proc/sys/fs/file-max"),
-        OFLAG_PROC,
-        Mode::empty(),
-    )?;
+    #[expect(clippy::disallowed_methods)]
+    let fd = nix::fcntl::openat(AT_BADFD, "/proc/sys/fs/file-max", OFLAG_PROC, Mode::empty())?;
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
@@ -793,13 +790,8 @@ pub fn proc_fs_file_max() -> Result<u64, Errno> {
 pub fn proc_fs_nr_open() -> Result<u64, Errno> {
     // SAFETY: Informational command,
     // runs before syd::config::proc_init.
-    #[allow(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(
-        AT_BADFD,
-        XPath::from_bytes(b"/proc/sys/fs/nr_open"),
-        OFLAG_PROC,
-        Mode::empty(),
-    )?;
+    #[expect(clippy::disallowed_methods)]
+    let fd = nix::fcntl::openat(AT_BADFD, "/proc/sys/fs/nr_open", OFLAG_PROC, Mode::empty())?;
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
@@ -812,10 +804,10 @@ pub fn proc_fs_nr_open() -> Result<u64, Errno> {
 pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
     // SAFETY: Informational command,
     // runs before syd::config::proc_init.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = nix::fcntl::openat(
         AT_BADFD,
-        XPath::from_bytes(b"/proc/sys/vm/mmap_min_addr"),
+        "/proc/sys/vm/mmap_min_addr",
         OFLAG_PROC,
         Mode::empty(),
     )?;
@@ -831,10 +823,10 @@ pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
 pub fn proc_pid_max(max: u64) -> Result<(), Errno> {
     // SAFETY: System configuration command,
     // runs during syd::config::proc_init.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = nix::fcntl::openat(
         AT_BADFD,
-        XPath::from_bytes(b"/proc/sys/kernel/pid_max"),
+        "/proc/sys/kernel/pid_max",
         OFLAG_PROC | OFlag::O_WRONLY,
         Mode::empty(),
     )?;
@@ -844,14 +836,13 @@ pub fn proc_pid_max(max: u64) -> Result<(), Errno> {
 }
 
 /// Retrieves the value of /proc/sys/kernel/randomize_va_space
-#[allow(clippy::disallowed_methods)]
 pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
     // SAFETY: Informational command,
     // runs before syd::config::proc_init.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = nix::fcntl::openat(
         AT_BADFD,
-        XPath::from_bytes(b"/proc/sys/kernel/randomize_va_space"),
+        "/proc/sys/kernel/randomize_va_space",
         OFLAG_PROC,
         Mode::empty(),
     )?;
@@ -864,9 +855,9 @@ pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
 }
 
 /// Reads the tty number from /proc/[pid]/stat and figures out the corresponding /dev/tty device node path.
-#[allow(clippy::arithmetic_side_effects)]
-#[allow(clippy::cast_sign_loss)]
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::arithmetic_side_effects)]
+#[expect(clippy::cast_sign_loss)]
+#[expect(clippy::disallowed_methods)]
 pub fn proc_tty(pid: Pid) -> Result<XPathBuf, Errno> {
     let stat = proc_stat(pid)?;
     if stat.tty_nr <= 0 {
@@ -999,7 +990,7 @@ pub fn proc_task_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
             Err(errno) => return Err(errno),
         };
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         for entry in &mut entries {
             // PID paths are directories.
             if !entry.is_dir() {
@@ -1028,7 +1019,7 @@ pub fn proc_task_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
 
     // Count tasks per-process.
     for task in tasks {
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         match proc_task_nr(Pid::from_raw(task)) {
             Ok(n) => count += n,
             Err(_) => continue, // task died mid-way?
@@ -1115,7 +1106,7 @@ pub fn proc_mem_limit(pid: Pid, max: u64) -> Result<bool, Errno> {
 /// for all unique paths with executable permissions along with their
 /// device ID and inode. On error, it returns an `Errno`. If no executable
 /// is found in maps, returns `Errno::ENOEXEC`.
-#[allow(clippy::type_complexity)]
+#[expect(clippy::type_complexity)]
 pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errno> {
     if *crate::config::HAVE_PROCMAP_QUERY {
         // Use the efficient PROCMAP_QUERY ioctl(2) on Linux>=6.11.
@@ -1158,7 +1149,7 @@ pub fn proc_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errn
 /// for all unique paths with executable permissions along with their
 /// inode and device numbers. On error, returns an `Errno`. If no executable
 /// mappings are found, returns `Errno::ENOEXEC`.
-#[allow(clippy::type_complexity)]
+#[expect(clippy::type_complexity)]
 pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i32)>, Errno> {
     // Open /proc/$pid/maps.
     let maps = proc_maps_open(pid)?;
@@ -1186,7 +1177,7 @@ pub fn procmap_query_executables(pid: Pid) -> Result<Vec<(XPathBuf, u64, i32, i3
                     .map(|cstr| cstr.to_bytes())
                     .map(XPathBuf::from)
                     .map_err(|_| Errno::EINVAL)?;
-                #[allow(clippy::cast_possible_wrap)]
+                #[expect(clippy::cast_possible_wrap)]
                 paths.insert((path, q.inode, q.dev_major as i32, q.dev_minor as i32));
 
                 addr = q.vma_end;
@@ -1347,7 +1338,7 @@ pub fn proc_namespaces(pid: Pid) -> Result<Namespaces, Errno> {
     let mut ns = XPathBuf::from("/proc");
     ns.push_pid(pid);
     ns.push(b"ns");
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let dirfd = nix::fcntl::openat(
         AT_BADFD,
         &ns,
@@ -1357,7 +1348,7 @@ pub fn proc_namespaces(pid: Pid) -> Result<Namespaces, Errno> {
     .map_err(proc_errno)?;
 
     // Read directory entries using getdents64 in a loop.
-    #[allow(clippy::disallowed_types)]
+    #[expect(clippy::disallowed_types)]
     let mut namespaces = std::collections::HashMap::new();
     loop {
         let mut entries = match getdents64(&dirfd, DIRENT_BUF_SIZE) {
@@ -1406,7 +1397,7 @@ pub fn proc_crypto() -> Result<CryptoTable, Errno> {
 
 /// Open a FD to _proc_crypto_(5) and return an OwnedFd.
 pub fn proc_crypto_open() -> Result<OwnedFd, Errno> {
-    safe_open_proc(XPath::from_bytes(b"crypto"))
+    safe_open_proc("crypto")
 }
 
 /// Parses /proc/crypto from given Fd and returns the table of crypto algorithms.
@@ -1414,10 +1405,15 @@ pub fn proc_crypto_read(fd: OwnedFd) -> Result<CryptoTable, Errno> {
     CryptoTable::from_buf_read(BufReader::new(File::from(fd))).or(Err(Errno::EPERM))
 }
 
-/// Parses /proc/net/unix and returns the inodes of all UNIX domain sockets.
-pub fn proc_unix_get_inodes() -> Result<SydHashSet<u64>, Errno> {
-    let path = XPath::from_bytes(b"net/unix");
-    let file = safe_open_proc(path).map(File::from).map(BufReader::new)?;
+/// Parses /proc/pid/net/unix, return inodes.
+///
+/// Returns the inodes of all UNIX domain sockets.
+pub fn proc_unix_inodes(pid: Pid) -> Result<SydHashSet<u64>, Errno> {
+    let mut path = XPathBuf::from_pid(pid);
+    path.push(b"net");
+    path.push(b"unix");
+
+    let file = safe_open_proc(&path).map(File::from).map(BufReader::new)?;
     Ok(UnixNetEntries::from_buf_read(file)
         .map(|e| e.0)
         .or(Err(Errno::EPERM))?
@@ -1630,19 +1626,18 @@ pub fn proc_set_at_secure2(
     // AT_UID -> AT_EUID -> AT_GID -> AT_EGID -> AT_SECURE
     // We also ensure no other key is ever present in this sequence.
     // The AT_NULL key indicates the end of the vector.
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     const AT_NULL: usize = libc::AT_NULL as usize;
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     const AT_UID: usize = libc::AT_UID as usize;
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     const AT_EUID: usize = libc::AT_EUID as usize;
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     const AT_GID: usize = libc::AT_GID as usize;
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     const AT_EGID: usize = libc::AT_EGID as usize;
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     const AT_SECURE: usize = libc::AT_SECURE as usize;
-    #[allow(clippy::cast_possible_truncation)]
     const AT_REQKEY: &[usize] = &[AT_UID, AT_EUID, AT_GID, AT_EGID, AT_SECURE];
     let mut required_index = 0;
 
@@ -1751,7 +1746,7 @@ pub fn proc_set_at_secure2(
 /// is the file descriptor number and the `XPathBuf` is the path it points to.
 ///
 /// Useful for debugging file descriptor leaks.
-#[allow(clippy::type_complexity)]
+#[expect(clippy::type_complexity)]
 pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
     let pid = pid.unwrap_or_else(getpid);
 
@@ -1775,7 +1770,7 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         };
 
         for entry in &mut entries {
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             if dot < 2 && entry.is_dot() {
                 dot += 1;
                 continue;
@@ -1824,7 +1819,7 @@ pub fn proc_rand_fd(pid: Pid) -> Result<RawFd, Errno> {
     // SAFETY: To make this file descriptor harder to spot by an
     // attacker we duplicate it to a random fd number.
     for _ in range.clone() {
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         let fd_rand = randint(range.clone())? as RawFd;
 
         // Check if the slot is free.
@@ -1956,14 +1951,13 @@ impl fmt::Display for KernelTaintFlags {
 }
 
 /// Retrieves the value of /proc/sys/kernel/tainted as KernelTaintFlags.
-#[allow(clippy::disallowed_methods)]
 pub fn proc_kernel_taint() -> Result<KernelTaintFlags, Errno> {
     // SAFETY: Informational command,
     // runs before syd::config::proc_init.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let fd = nix::fcntl::openat(
         AT_BADFD,
-        XPath::from_bytes(b"/proc/sys/kernel/tainted"),
+        "/proc/sys/kernel/tainted",
         OFLAG_PROC,
         Mode::empty(),
     )?;
