@@ -14,18 +14,20 @@ use std::{
 };
 
 use libc::{c_int, dup3, GRND_RANDOM};
+use memchr::memchr;
 use nix::{
     errno::Errno,
     fcntl::{OFlag, ResolveFlag},
     sys::resource::{getrlimit, Resource},
     unistd::{close, UnlinkatFlags},
-    NixPath,
 };
 
 use crate::{
     cookie::safe_unlinkat,
-    fs::{is_active_fd, retry_on_eintr, safe_open},
-    path::{XPath, XPathBuf, PATH_MAX},
+    fs::{is_active_fd, safe_open},
+    log::{now, Tm},
+    path::{XPathBuf, PATH_MAX},
+    retry::retry_on_eintr,
 };
 
 /// RAII guard that disables pthread cancellation for the current thread
@@ -293,18 +295,18 @@ pub fn duprand(oldfd: RawFd, mut flags: OFlag) -> Result<RawFd, Errno> {
 /// mkstemp(3) function the template here does not have to end with any
 /// number of `X` characters. The function appends an implementation
 /// defined number of random characters after `prefix`. `prefix` must
-/// not start with the `/` character and not be longer than `PATH_MAX`
+/// not contain the `/` character and not be longer than `PATH_MAX`
 /// characters long. It is OK for prefix to be empty.
 /// If `dirfd` supports the `O_TMPFILE` operation, an unnamed temporary
 /// file is created instead with `O_TMPFILE|O_EXCL`.
-pub fn mkstempat<Fd: AsFd>(dirfd: Fd, prefix: &XPath) -> Result<OwnedFd, Errno> {
+pub fn mkstempat<Fd: AsFd>(dirfd: Fd, prefix: &[u8]) -> Result<OwnedFd, Errno> {
     const MAX_TCOUNT: usize = 8;
     const SUFFIX_LEN: usize = 128;
     const CHARSET: &[u8] = b"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 
     // Step 1: Attempt to use O_TMPFILE|O_EXCL which is safer.
     let mut flags = OFlag::O_TMPFILE | OFlag::O_EXCL | OFlag::O_RDWR;
-    match safe_open(&dirfd, XPath::from_bytes(b"."), flags, ResolveFlag::empty()) {
+    match safe_open(&dirfd, c".", flags, ResolveFlag::empty()) {
         Ok(fd) => return Ok(fd),
         Err(Errno::EISDIR | Errno::ENOENT | Errno::EOPNOTSUPP) => {}
         Err(errno) => return Err(errno),
@@ -313,7 +315,7 @@ pub fn mkstempat<Fd: AsFd>(dirfd: Fd, prefix: &XPath) -> Result<OwnedFd, Errno>
     // Step 2: Fallback to random name generation.
     flags.remove(OFlag::O_TMPFILE);
     flags.insert(OFlag::O_CREAT);
-    if prefix.is_absolute() {
+    if memchr(b'/', prefix).is_some() {
         return Err(Errno::EINVAL);
     } else if prefix.len().saturating_sub(SUFFIX_LEN) > PATH_MAX {
         return Err(Errno::ENAMETOOLONG);
@@ -321,9 +323,8 @@ pub fn mkstempat<Fd: AsFd>(dirfd: Fd, prefix: &XPath) -> Result<OwnedFd, Errno>
 
     let mut attempts = 0;
     let mut rng_data = [0u8; SUFFIX_LEN];
-    #[expect(clippy::arithmetic_side_effects)]
     loop {
-        attempts += 1;
+        attempts = attempts.checked_add(1).ok_or(Errno::EOVERFLOW)?;
         if attempts > MAX_TCOUNT {
             // Too many collisions.
             return Err(Errno::EEXIST);
@@ -333,10 +334,19 @@ pub fn mkstempat<Fd: AsFd>(dirfd: Fd, prefix: &XPath) -> Result<OwnedFd, Errno>
         fillrandom(&mut rng_data)?;
 
         // Map bytes to characters.
-        let mut base = XPathBuf::with_capacity(prefix.len() + SUFFIX_LEN);
-        base.append_bytes(prefix.as_bytes());
+        let size = prefix
+            .len()
+            .checked_add(SUFFIX_LEN)
+            .ok_or(Errno::EOVERFLOW)?;
+        let mut base = XPathBuf::new();
+        base.try_reserve(size).or(Err(Errno::ENOMEM))?;
+        base.append_bytes(prefix);
         for &b in &rng_data {
-            base.append_byte(CHARSET[(b as usize) % CHARSET.len()]);
+            let idx = (b as usize)
+                .checked_rem(CHARSET.len())
+                .ok_or(Errno::EOVERFLOW)?;
+            let chr = CHARSET.get(idx).copied().ok_or(Errno::EOVERFLOW)?;
+            base.append_byte(chr);
         }
 
         match safe_open(&dirfd, &base, flags, ResolveFlag::empty()) {
@@ -353,6 +363,53 @@ pub fn mkstempat<Fd: AsFd>(dirfd: Fd, prefix: &XPath) -> Result<OwnedFd, Errno>
     }
 }
 
+/// Generate a random Linux kernel version string.
+pub fn rand_version() -> Result<String, Errno> {
+    const VERMAGICS: &[&str] = &[
+        "SMP",
+        "SMP PREEMPT",
+        "SMP PREEMPT_DYNAMIC",
+        "SMP PREEMPT_RT",
+    ];
+    const MONTHS: &[&str] = &[
+        "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
+    ];
+    const WKDAYS: &[&str] = &["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
+
+    // Subtract a random number of seconds within ~last year.
+    const TS_WINDOW: u64 = 366 * 86_400;
+    let now = now();
+    let offset: u64 = randint(0..=TS_WINDOW)?;
+    let target: i64 = now
+        .saturating_sub(offset)
+        .try_into()
+        .or(Err(Errno::EOVERFLOW))?;
+
+    // Break down that instant.
+    let tm = Tm::try_from(target)?;
+
+    // Randomize build number and PREEMPT variant.
+    let build_no = randint(1u8..=64)?;
+    #[expect(clippy::arithmetic_side_effects)]
+    let vermagic = VERMAGICS[randint(0usize..=(VERMAGICS.len() - 1))?];
+
+    // Determine version month and day.
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_sign_loss)]
+    let mon = MONTHS[(tm.month() - 1) as usize];
+    #[expect(clippy::cast_sign_loss)]
+    let wday = WKDAYS[tm.weekday() as usize];
+    let mday = tm.day();
+    let hh = tm.hour();
+    let mm = tm.minute();
+    let ss = tm.second();
+    let year = tm.year();
+
+    Ok(format!(
+        "#{build_no} {vermagic} {wday} {mon} {mday:>2} {hh:02}:{mm:02}:{ss:02} UTC {year}",
+    ))
+}
+
 #[cfg(test)]
 mod tests {
     use std::fmt::Debug;
