@@ -32,10 +32,9 @@ use nix::{
         c_int, c_uint, F_SEAL_FUTURE_WRITE, F_SEAL_GROW, F_SEAL_SEAL, F_SEAL_SHRINK, F_SEAL_WRITE,
         MFD_ALLOW_SEALING, MFD_CLOEXEC, MFD_EXEC, MFD_NOEXEC_SEAL,
     },
-    sys::memfd::MFdFlags,
 };
 
-use crate::err::err2no;
+use crate::{compat::MFdFlags, err::err2no};
 
 // SAFETY: The provided slice is nul-terminated and does not contain any interior nul bytes. On Rust
 // 1.64 and later (rust-lang/rust#99977), these required invariants are checked at compile time.
@@ -76,7 +75,7 @@ const OPTIONS: SealOptions = SealOptions::new().close_on_exec(true).executable(t
 pub fn ensure_sealed() -> Result<(), Errno> {
     // SAFETY: This runs before sandboxing starts,
     // so safe open API is not available yet.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let mut file = File::open("/proc/self/exe").map_err(|err| err2no(&err))?;
     if OPTIONS.is_sealed(&file) {
         Ok(())
@@ -482,7 +481,7 @@ impl SealOptions {
 }
 
 fn memfd_create(name: &CStr, flags: c_uint) -> Result<File, Errno> {
-    nix::sys::memfd::memfd_create(name, MFdFlags::from_bits_retain(flags)).map(File::from)
+    nix::sys::memfd::memfd_create(name, MFdFlags::from_bits_retain(flags).into()).map(File::from)
 }
 
 fn fcntl_get_seals(file: &File) -> Result<c_int, Errno> {
