@@ -121,8 +121,8 @@ pub struct SealBox<T> {
 impl<T> SealBox<T> {
     /// Allocate and initialize `data` on a page-aligned, read/write anonymous mapping.
     pub fn new(data: T) -> Result<Self, Errno> {
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
         let page = sysconf(SysconfVar::PAGE_SIZE)?.ok_or(Errno::EINVAL)? as usize;
 
         let size = mem::size_of::<T>();
@@ -211,8 +211,8 @@ impl<T> SealBox<T> {
 impl<T> SealBox<MaybeUninit<T>> {
     /// Allocate space for a `T` but do *not* initialize it.
     pub fn new_uninit() -> Result<Self, Errno> {
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
         let page = sysconf(SysconfVar::PAGE_SIZE)?.ok_or(Errno::EINVAL)? as usize;
 
         let size = mem::size_of::<T>();
@@ -323,8 +323,8 @@ impl<E> SealBoxSlice<E> {
     {
         let len = slice.len();
 
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
         let page = sysconf(SysconfVar::PAGE_SIZE)?.ok_or(Errno::EINVAL)? as usize;
 
         if len == 0 {
@@ -837,6 +837,20 @@ impl<T> SealableVec<T> {
         }
     }
 
+    /// Retain only elements matching `f` if unsealed, else return `Err(EPERM)`.
+    pub fn retain_mut<F>(&mut self, f: F) -> Result<(), Errno>
+    where
+        F: FnMut(&mut T) -> bool,
+    {
+        match self {
+            SealableVec::Unsealed(v) => {
+                v.retain_mut(f);
+                Ok(())
+            }
+            SealableVec::Sealed(_) => Err(Errno::EPERM),
+        }
+    }
+
     /// Extend from `iter` if unsealed, else return `Err(EPERM)`.
     pub fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) -> Result<(), Errno> {
         match self {
@@ -1117,6 +1131,20 @@ impl<T> SealableVecDeque<T> {
         }
     }
 
+    /// Retain only elements matching `f` if unsealed, else return `Err(EPERM)`.
+    pub fn retain_mut<F>(&mut self, f: F) -> Result<(), Errno>
+    where
+        F: FnMut(&mut T) -> bool,
+    {
+        match self {
+            SealableVecDeque::Unsealed(v) => {
+                v.retain_mut(f);
+                Ok(())
+            }
+            SealableVecDeque::Sealed(_) => Err(Errno::EPERM),
+        }
+    }
+
     /// Extend from `iter` if unsealed, else return `Err(EPERM)`.
     pub fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) -> Result<(), Errno> {
         match self {
