@@ -13,202 +13,219 @@ use std::{
     time::Instant,
 };
 
+use libseccomp::ScmpArch;
 use nix::errno::Errno;
 
 use crate::{
     config::*,
-    fs::{create_memfd, fillrandom, randint, seal_memfd, MFD_ALLOW_SEALING, MFD_NOEXEC_SEAL},
+    cookie::safe_memfd_create,
+    fs::seal_memfd,
+    rng::{fillrandom, randint},
 };
 
+#[repr(C)]
+#[derive(Clone, Copy, Debug)]
+pub(crate) struct sysinfo32 {
+    uptime: i32,
+    loads: [u32; 3],
+    totalram: u32,
+    freeram: u32,
+    sharedram: u32,
+    bufferram: u32,
+    totalswap: u32,
+    freeswap: u32,
+    procs: u16,
+    pad: u16,
+    totalhigh: u32,
+    freehigh: u32,
+    mem_unit: u32,
+    _f: [u8; 8],
+}
+
+#[repr(C)]
+#[derive(Clone, Copy, Debug)]
+pub(crate) struct sysinfo64 {
+    uptime: i64,
+    loads: [u64; 3],
+    totalram: u64,
+    freeram: u64,
+    sharedram: u64,
+    bufferram: u64,
+    totalswap: u64,
+    freeswap: u64,
+    procs: u16,
+    pad: u16,
+    totalhigh: u64,
+    freehigh: u64,
+    mem_unit: u32,
+    _f: [u8; 0],
+}
+
 /// Randomized sysinfo(2)
-pub struct SysInfo(libc::sysinfo);
+pub(crate) enum SysInfo {
+    S32(sysinfo32),
+    S64(sysinfo64),
+}
 
-impl SysInfo {
-    /// Initialize a new randomized sysinfo(2) structure.
-    // TODO: Use checked arithmetic!
-    #[allow(clippy::arithmetic_side_effects)]
-    pub fn new() -> Result<Self, Errno> {
-        // SAFETY: `libc::sysinfo` is a POD (plain old data) type, so
-        // zeroing it out does not violate any invariants. We do this
-        // to ensure it's initialized to a known state before
-        // randomizing.
-        let mut info: libc::sysinfo = unsafe { std::mem::zeroed() };
+#[inline]
+fn fill_pod_random<T>(pod: &mut T) -> Result<(), Errno> {
+    let siz = std::mem::size_of::<T>();
+    let ptr = (pod as *mut T) as *mut u8;
+    // SAFETY: ptr is a valid sysinfo32 or sysinfo64.
+    let bytes = unsafe { std::slice::from_raw_parts_mut(ptr, siz) };
+    fillrandom(bytes)
+}
 
-        // Randomize the struct memory in 256-byte batches to avoid EINVAL from fillrandom.
-        let info_siz = std::mem::size_of::<libc::sysinfo>();
-        let info_ptr = std::ptr::addr_of_mut!(info) as *mut u8;
-        let mut offset = 0_usize;
-        while offset < info_siz {
-            let end = (offset + 256).min(info_siz);
-            // SAFETY: We create a slice covering a valid portion
-            // (offset..end) of `info`'s memory. This is safe because
-            // offset and end are kept within the size of `info`.
-            let slice =
-                unsafe { std::slice::from_raw_parts_mut(info_ptr.add(offset), end - offset) };
-            fillrandom(slice)?;
-            offset = end;
-        }
+macro_rules! init_sysinfo {
+    ($info:ident, $U:ty, $S:ty) => {{
+        // randomize entire struct first
+        fill_pod_random(&mut $info)?;
 
-        // SAFETY: Use realistic values to avoid easy detection.
-        info.mem_unit = 1;
+        // fixed fields
+        $info.mem_unit = 1;
+        $info.totalhigh = 0 as $U;
+        $info.freehigh = 0 as $U;
+        $info.totalswap = 0 as $U;
+        $info.freeswap = 0 as $U;
 
-        info.totalhigh = 0;
-        info.freehigh = 0;
+        // realistic memory limits (unchanged)
+        const MIN_RAM: $U = 0x0080_0000 as $U; // 128 MiB
+        const MAX_RAM: $U = 0xFFFF_FFFF as $U; // 4 GiB
 
-        info.totalswap = 0;
-        info.freeswap = 0;
-
-        // Define realistic memory size limits.
-        const MIN_RAM: libc::c_ulong = 0x8000000; // 128 MB
-        const MAX_RAM: libc::c_ulong = 0xffffffff; // 4 GB
-
-        // Adjust `totalram` to be a power of two within realistic limits.
-        let mut totalram = info.totalram % (MAX_RAM - MIN_RAM + 1) + MIN_RAM;
+        // totalram: pow2 within [MIN_RAM, MAX_RAM]
+        let mut totalram: $U = $info.totalram % (MAX_RAM - MIN_RAM + 1 as $U) + MIN_RAM;
         totalram = if totalram.is_power_of_two() {
             totalram
         } else {
             totalram.checked_next_power_of_two().unwrap_or(MAX_RAM) >> 1
         };
         totalram = totalram.clamp(MIN_RAM, MAX_RAM);
-        info.totalram = totalram;
+        $info.totalram = totalram;
 
-        // Adjust `freeram` to be a power of two ≤ `totalram`.
-        let mut freeram = info.freeram % (info.totalram + 1);
+        // freeram ≤ totalram, pow2
+        let mut freeram: $U = $info.freeram % ($info.totalram + 1 as $U);
         freeram = if freeram.is_power_of_two() {
             freeram
         } else {
-            freeram.checked_next_power_of_two().unwrap_or(info.totalram) >> 1
+            freeram
+                .checked_next_power_of_two()
+                .unwrap_or($info.totalram)
+                >> 1
         };
-        freeram = freeram.min(info.totalram);
-        info.freeram = freeram;
+        $info.freeram = freeram.min($info.totalram);
 
-        // Adjust `sharedram` to be a power of two ≤ `totalram`.
-        let mut sharedram = info.sharedram % (info.totalram + 1);
+        // sharedram ≤ totalram, pow2
+        let mut sharedram: $U = $info.sharedram % ($info.totalram + 1 as $U);
         sharedram = if sharedram.is_power_of_two() {
             sharedram
         } else {
             sharedram
                 .checked_next_power_of_two()
-                .unwrap_or(info.totalram)
+                .unwrap_or($info.totalram)
                 >> 1
         };
-        sharedram = sharedram.min(info.totalram);
-        info.sharedram = sharedram;
+        $info.sharedram = sharedram.min($info.totalram);
 
-        // Adjust `bufferram` to be a power of two ≤ `totalram`.
-        let mut bufferram = info.bufferram % (info.totalram + 1);
+        // bufferram ≤ totalram, pow2
+        let mut bufferram: $U = $info.bufferram % ($info.totalram + 1 as $U);
         bufferram = if bufferram.is_power_of_two() {
             bufferram
         } else {
             bufferram
                 .checked_next_power_of_two()
-                .unwrap_or(info.totalram)
+                .unwrap_or($info.totalram)
                 >> 1
         };
-        bufferram = bufferram.min(info.totalram);
-        info.bufferram = bufferram;
-
-        // SAFETY: Use the global RAND_TIMER for consistent timing.
-        // Note, uptime is `long` in glibc and `unsigned long` in musl.
-        #[cfg(target_env = "musl")]
-        {
-            info.uptime = RAND_TIMER().uptime() as libc::c_ulong;
-        }
-        #[cfg(not(target_env = "musl"))]
-        {
-            info.uptime = RAND_TIMER().uptime() as libc::c_long;
-        }
-
-        // Load averages are fixed-point numbers with scaling factor 65536.
-        const LOAD_SCALE: libc::c_ulong = 0x10000;
-        const MAX_LOAD: libc::c_ulong = LOAD_SCALE * 16; // Max load average of 16.0
-        for load in &mut info.loads {
-            *load %= MAX_LOAD + 1;
-        }
-
-        // Adjust `procs` to be within a realistic range.
-        const MIN_PROCS: libc::c_ushort = 2;
-        const MAX_PROCS: libc::c_ushort = 0x8000;
-        info.procs = (info.procs % (MAX_PROCS - MIN_PROCS + 1)) + MIN_PROCS;
-
-        // Zero out the padding for security.
-        info.pad = 0;
-
-        Ok(Self(info))
-    }
-
-    /// Return the inner sysinfo structure.
-    pub fn as_raw(self) -> libc::sysinfo {
-        self.0
-    }
-
-    /// Produces a randomized string mimicking proc_loadavg(5), e.g:
-    ///
-    /// `"<load1> <load5> <load15> <running>/<procs> <last_pid>"`
-    ///
-    /// Loads come from `self.0.loads`.  The `running` count is random
-    /// up to `self.0.procs`.  The `last_pid` is also random, in a
-    /// plausible range (e.g. 256..65535).
-    ///
-    /// Example output:
-    /// 2.64 2.28 2.26 3/541 1232852
-    #[allow(clippy::cast_precision_loss)]
-    pub fn proc_loadavg(&self) -> Result<String, Errno> {
-        // Linux kernel scaling factor for load averages: 65536
-        let load_scale = 65536.0;
-
-        // Convert each load from fixed-point to float
-        let load1 = self.0.loads[0] as f64 / load_scale;
-        let load5 = self.0.loads[1] as f64 / load_scale;
-        let load15 = self.0.loads[2] as f64 / load_scale;
-
-        // `procs` is the total number of processes.
-        // We'll pick a random "running" count from 1..=procs (if procs > 0).
-        // If procs is 0 (extreme corner case), we'll treat running as 0.
-        let procs = self.0.procs;
+        $info.bufferram = bufferram.min($info.totalram);
 
+        // uptime (same source, cast to signed kernel long width)
         #[allow(clippy::cast_possible_truncation)]
-        let running = if procs == 0 {
-            0
-        } else {
-            // Cap the random value at procs, ensuring running <= procs
-            let r = randint(1..=u64::from(procs))?;
-            r.min(u64::from(procs)) as u16
-        };
+        #[allow(clippy::cast_possible_wrap)]
+        {
+            $info.uptime = RAND_TIMER().uptime() as $S;
+        }
 
-        // Choose a plausible last_pid in the range [256..63535].
-        let last_pid = randint(256..=0xFFFF)?;
+        // loads: fixed-point / 65536, cap at 16.0
+        const LOAD_SCALE: $U = 0x1_0000 as $U;
+        const MAX_LOAD: $U = (LOAD_SCALE * 16) as $U;
+        for ld in &mut $info.loads {
+            *ld %= MAX_LOAD + 1 as $U;
+        }
 
-        // Format like proc_loadavg(5).
-        Ok(format!(
-            "{load1:.2} {load5:.2} {load15:.2} {running}/{procs} {last_pid}\n"
-        ))
+        // procs in a realistic range
+        const MIN_PROCS: u16 = 2;
+        const MAX_PROCS: u16 = 0x8000;
+        $info.procs = ($info.procs % (MAX_PROCS - MIN_PROCS + 1)) + MIN_PROCS;
+
+        Ok::<(), Errno>(())
+    }};
+}
+
+impl SysInfo {
+    /// Initialize a new randomized sysinfo(2) structure.
+    #[inline]
+    pub fn new(arch: ScmpArch) -> Result<Self, Errno> {
+        match arch {
+            // 64-bit kernel long/ulong layouts
+            ScmpArch::X8664
+            | ScmpArch::X32 // x32 uses 64-bit __kernel_[u]long_t
+            | ScmpArch::Aarch64
+            | ScmpArch::Mips64
+            | ScmpArch::Mipsel64
+            | ScmpArch::Ppc64
+            | ScmpArch::Ppc64Le
+            | ScmpArch::Riscv64
+            | ScmpArch::S390X
+            | ScmpArch::Loongarch64 => Self::new64(),
+
+            // 32-bit kernel long/ulong layouts
+            ScmpArch::X86
+            | ScmpArch::Arm
+            | ScmpArch::M68k
+            | ScmpArch::Mips
+            | ScmpArch::Mipsel
+            | ScmpArch::Mips64N32     // ILP32 on 64-bit mips
+            | ScmpArch::Mipsel64N32   // ILP32 on 64-bit mips (little-endian)
+            | ScmpArch::S390          // 31-bit compat personality
+            => Self::new32(),
+            _ => Err(Errno::ENOSYS),
+        }
     }
 
-    /// Returns a /proc/uptime compatible memory file descriptor.
-    ///
-    /// The memory file descriptor is write-sealed.
-    /// The memory file descriptor is exec-sealed if restrict_memfd is true.
-    pub fn proc_loadavg_fd(&self, restrict_memfd: bool) -> Result<OwnedFd, Errno> {
-        let repr = self.proc_loadavg()?;
-        let data = repr.as_bytes();
+    /// Initialize a new randomized sysinfo(2) structure for 32-bit.
+    pub(crate) fn new32() -> Result<Self, Errno> {
+        // SAFETY: sysinfo32 is repr(C) and zero-initialization is valid for this type.
+        let mut info: sysinfo32 = unsafe { std::mem::zeroed() };
+        init_sysinfo!(info, u32, i32)?;
+        Ok(SysInfo::S32(info))
+    }
 
-        // Note, MFD_ALLOW_SEALING is implied for MFD_NOEXEC_SEAL.
-        let flags = if restrict_memfd {
-            MFD_NOEXEC_SEAL
-        } else {
-            MFD_ALLOW_SEALING
-        };
-        let fd = create_memfd(b"syd-loadavg\0", flags)?;
-        let mut file = File::from(fd);
+    /// Initialize a new randomized sysinfo(2) structure for 64-bit.
+    pub fn new64() -> Result<Self, Errno> {
+        // SAFETY: sysinfo64 is repr(C) and zero-initialization is valid for this type.
+        let mut info: sysinfo64 = unsafe { std::mem::zeroed() };
+        init_sysinfo!(info, u64, i64)?;
+        Ok(SysInfo::S64(info))
+    }
 
-        file.write_all(data).or(Err(Errno::EIO))?;
-        file.rewind().or(Err(Errno::EIO))?;
-
-        // SAFETY: Deny further writes to the file descriptor.
-        seal_memfd(&file)?;
-
-        Ok(file.into())
+    #[inline]
+    pub fn as_bytes(&self) -> &[u8] {
+        match self {
+            // SAFETY: s is a valid sysinfo32 and we only read its memory as bytes.
+            Self::S32(s) => unsafe {
+                std::slice::from_raw_parts(
+                    (s as *const _) as *const u8,
+                    std::mem::size_of::<sysinfo32>(),
+                )
+            },
+            // SAFETY: s is a valid sysinfo64 and we only read its memory as bytes.
+            Self::S64(s) => unsafe {
+                std::slice::from_raw_parts(
+                    (s as *const _) as *const u8,
+                    std::mem::size_of::<sysinfo64>(),
+                )
+            },
+        }
     }
 }
 
@@ -263,18 +280,12 @@ impl RandTimer {
     /// Returns a /proc/uptime compatible memory file descriptor.
     ///
     /// The memory file descriptor is write-sealed.
-    /// The memory file descriptor is exec-sealed if restrict_memfd is true.
-    pub fn proc_fd(&self, restrict_memfd: bool) -> Result<OwnedFd, Errno> {
+    /// The memory file descriptor is exec-sealed on Linux>=6.3.
+    pub fn proc_fd(&self) -> Result<OwnedFd, Errno> {
         let repr = self.proc();
         let data = repr.as_bytes();
 
-        // Note, MFD_ALLOW_SEALING is implied for MFD_NOEXEC_SEAL.
-        let flags = if restrict_memfd {
-            MFD_NOEXEC_SEAL
-        } else {
-            MFD_ALLOW_SEALING
-        };
-        let fd = create_memfd(b"syd-uptime\0", flags)?;
+        let fd = safe_memfd_create(c"syd-proc-uptime", *SAFE_MFD_FLAGS)?;
         let mut file = File::from(fd);
 
         file.write_all(data).or(Err(Errno::EIO))?;
