@@ -25,7 +25,9 @@ use nix::{
 use crate::{
     compat::{waitid, WaitStatus},
     config::CHLD_STACK_SIZE,
-    fs::{duprand, pidfd_getfd, retry_on_eintr},
+    fs::pidfd_getfd,
+    retry::retry_on_eintr,
+    rng::duprand,
     unshare::{child, config::Config, Child, Command, Executable, IoctlMap},
 };
 
@@ -156,7 +158,6 @@ impl Command {
         }
     }
 
-    #[allow(clippy::cognitive_complexity)]
     fn after_start<Fd: AsFd>(mut self, pid: Pid, pid_fd: Fd) -> Result<RawFd, Errno> {
         if self.config.stop {
             // Seize the process for tracing.
@@ -181,12 +182,20 @@ impl Command {
             )?;
             assert_eq!(status, WaitStatus::Stopped(pid, libc::SIGSTOP));
             // Step 2: Seize the process.
-            // We use PTRACE_SEIZE in the parent rather than
-            // PTRACE_TRACEME in the child for its improved
-            // behaviour/API.  This also gives us the chance to deny
-            // PTRACE_TRACEME and further confine the sandbox against
-            // e.g. trivial ptrace detectors.
-            seize(pid, ptrace_options)?;
+            // 1. We use PTRACE_SEIZE in the parent rather than
+            //    PTRACE_TRACEME in the child for its improved
+            //    behaviour/API. This also gives us the chance to deny
+            //    PTRACE_TRACEME and further confine the sandbox against
+            //    e.g. trivial ptrace detectors.
+            // 2. Panic if PTRACE_SEIZE fails as otherwise we will leave
+            //    the sandbox process in an uninterruptible, broken state.
+            //    The typical error case is EPERM which means parent is
+            //    strace or YAMA is active.
+            assert_eq!(
+                seize(pid, ptrace_options),
+                Ok(()),
+                "YAMA or strace? Use with strace -f syd -pD ..."
+            );
             let status = waitid(
                 Id::PIDFd(pid_fd.as_fd()),
                 WaitPidFlag::WEXITED | WaitPidFlag::WSTOPPED | WaitPidFlag::__WNOTHREAD,
@@ -237,7 +246,7 @@ impl Command {
         // Landlock signal scopes are new in ABI 6, new in Linux>=6.12.
 
         if let Some(ref mut callback) = self.before_unfreeze {
-            #[allow(clippy::cast_sign_loss)]
+            #[expect(clippy::cast_sign_loss)]
             callback(i32::from(pid) as u32)?;
         }
 
@@ -271,7 +280,7 @@ impl Command {
         let mut buf = vec![0u8; std::mem::size_of::<RawFd>()];
         let mut nread = 0;
         while nread < buf.len() {
-            #[allow(clippy::arithmetic_side_effects)]
+            #[expect(clippy::arithmetic_side_effects)]
             match read(&pipe_ro, &mut buf[nread..]) {
                 Ok(0) => return Err(Errno::EIO),
                 Ok(n) => nread += n,
