@@ -11,12 +11,11 @@ use std::{fs::File, os::fd::AsRawFd, process::ExitCode};
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::errno::Errno;
 use syd::{
-    confine::confine_mdwe,
+    confine::{confine_mdwe, confine_scmp_madvise, confine_scmp_wx},
     elf::{ElfError, ElfType, ExecutableFile, LinkingType},
     err::SydResult,
     landlock_policy::LandlockPolicy,
     path::XPathBuf,
-    proc::proc_mmap_min_addr,
     syslog::LogLevel,
 };
 
@@ -208,93 +207,11 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     let _ = confine_mdwe(false);
 
     // W^X filter allows by default and kills offending memory access.
-    let mut ctx = ScmpFilterContext::new(ScmpAction::Allow)?;
-
-    // Enforce the NO_NEW_PRIVS functionality before
-    // loading the seccomp filter into the kernel.
-    ctx.set_ctl_nnp(true)?;
-
-    // Kill process for bad arch.
-    ctx.set_act_badarch(ScmpAction::KillProcess)?;
-
-    // We don't want ECANCELED, we want actual errnos.
-    ctx.set_api_sysrawrc(true)?;
-
-    // Use a binary tree sorted by syscall number, if possible.
-    let _ = ctx.set_ctl_optimize(2);
-
-    // Restriction -1: Prevent mmap(addr<${mmap_min_addr}, MAP_FIXED).
-    const MAP_FIXED: u64 = nix::libc::MAP_FIXED as u64;
-    const MAP_FIXED_NOREPLACE: u64 = nix::libc::MAP_FIXED_NOREPLACE as u64;
-    let mmap_min_addr = proc_mmap_min_addr().unwrap_or(4096);
-    for sysname in ["mmap", "mmap2"] {
-        #[expect(clippy::disallowed_methods)]
-        let syscall = ScmpSyscall::from_name(sysname).unwrap();
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[
-                scmp_cmp!($arg0 < mmap_min_addr),
-                scmp_cmp!($arg3 & MAP_FIXED == MAP_FIXED),
-            ],
-        )?;
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[
-                scmp_cmp!($arg0 < mmap_min_addr),
-                scmp_cmp!($arg3 & MAP_FIXED_NOREPLACE == MAP_FIXED_NOREPLACE),
-            ],
-        )?;
-    }
-
-    // Restriction 0: Prohibit attempts to create memory mappings
-    // that are writable and executable at the same time, or to
-    // change existing memory mappings to become executable, or
-    // mapping shared memory segments as executable.
-    const W: u64 = nix::libc::PROT_WRITE as u64;
-    const X: u64 = nix::libc::PROT_EXEC as u64;
-    const WX: u64 = W | X;
-    const MAP_S: u64 = nix::libc::MAP_SHARED as u64;
-    for sysname in ["mmap", "mmap2"] {
-        // Prevent writable and executable memory.
-        #[expect(clippy::disallowed_methods)]
-        let syscall = ScmpSyscall::from_name(sysname).unwrap();
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[scmp_cmp!($arg2 & WX == WX)],
-        )?;
-
-        // Prevent executable shared memory.
-        ctx.add_rule_conditional(
-            ScmpAction::KillProcess,
-            syscall,
-            &[scmp_cmp!($arg2 & X == X), scmp_cmp!($arg3 & MAP_S == MAP_S)],
-        )?;
-    }
-
-    ctx.add_rule_conditional(
-        ScmpAction::KillProcess,
-        ScmpSyscall::from_name("mprotect")?,
-        &[scmp_cmp!($arg2 & X == X)],
-    )?;
-
-    // All set, load the seccomp filter.
     //
-    // SAFETY: Ignore EINVAL which means at least one of:
+    // Ignore errors which may mean at least one of:
     // a. CONFIG_SECCOMP_FILTER not enabled in kernel.
     // b. Syd is denying stacked seccomp cbpf filters.
-    if let Err(error) = ctx.load() {
-        if error
-            .sysrawrc()
-            .map(|errno| errno.abs())
-            .unwrap_or(libc::ECANCELED)
-            != libc::EINVAL
-        {
-            return Err(error.into());
-        }
-    }
+    let _ = confine_scmp_wx();
 
     // Step 3: Confine system calls to least privilege using seccomp-bpf.
     let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
@@ -317,7 +234,7 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
         "brk",
         "exit",
         "exit_group",
-        "madvise",
+        //"madvise", advice are confined.
         "mmap",
         "mmap2",
         "mprotect",
@@ -330,6 +247,9 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
         ctx.add_rule(ScmpAction::Allow, ScmpSyscall::from_name(sysname)?)?;
     }
 
+    // Allow safe madvise(2) advice.
+    confine_scmp_madvise(&mut ctx)?;
+
     // Allow read, seek, close of file.
     let fd = fd.as_raw_fd() as u64;
     for sysname in ["close", "read", "readv", "_llseek", "lseek"] {
