@@ -20,6 +20,16 @@ use syd::{
     confine::{confine_mdwe, confine_scmp_wx, run_cmd},
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -57,6 +67,8 @@ syd::main! {
         }
     }
 
+    // PR_SET_MDWE is fatal only if -m is given.
+    let opt_mdwe_fatal = opt_mdwe;
     if !opt_mdwe && !opt_scmp {
         // Default is to enable both.
         opt_mdwe = true;
@@ -66,7 +78,9 @@ syd::main! {
     if opt_mdwe {
         if let Err(errno) = confine_mdwe(false) {
             eprintln!("prctl failed to set Memory-Deny-Write-Execute: {errno}!");
-            return Err(errno.into());
+            if opt_mdwe_fatal {
+                return Err(errno.into());
+            }
         }
     }
 
