@@ -21,15 +21,27 @@ use nix::{
     unistd::{isatty, Gid, Uid},
 };
 use syd::{
+    compat::MFdFlags,
     config::*,
+    cookie::safe_memfd_create,
     debug,
-    fs::{create_memfd, safe_copy_if_exists, seal_memfd, set_cloexec, MFD_ALLOW_SEALING},
+    fs::{safe_copy_if_exists, seal_memfd, set_cloexec},
     get_user_home, get_user_name, human_size,
-    path::{XPath, XPathBuf},
+    path::XPathBuf,
     syslog::LogLevel,
     wordexp::*,
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -87,7 +99,9 @@ syd::main! {
     // Create a memory fd to write input into,
     // and pass to the internal /bin/sh invoked
     // by wordexp(3).
-    let mut file = create_memfd(b"syd-sh\0", MFD_ALLOW_SEALING).map(File::from)?;
+    let mut file = safe_memfd_create(
+        c"syd-sh",
+        MFdFlags::MFD_ALLOW_SEALING | MFdFlags::MFD_CLOEXEC).map(File::from)?;
     debug!("ctx": "sh",
         "msg": format!("created memory-file {} with close-on-exec flag set",
             file.as_raw_fd()));
@@ -106,10 +120,10 @@ syd::main! {
 
     // Handle system-wide configuration.
     if optl {
-        safe_copy_if_exists(&mut file, XPath::from_bytes(b"/etc/syd/init_login.sh"))?;
+        safe_copy_if_exists(&mut file, "/etc/syd/init_login.sh")?;
         file.write_all(b"\n")?;
     }
-    safe_copy_if_exists(&mut file, XPath::from_bytes(b"/etc/syd/init.sh"))?;
+    safe_copy_if_exists(&mut file, "/etc/syd/init.sh")?;
     file.write_all(b"\n")?;
 
     // Handle user-specific configuration.
@@ -183,8 +197,8 @@ syd::main! {
         };
     }
 
-    #[allow(clippy::type_complexity)]
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::type_complexity)]
+    #[expect(clippy::disallowed_methods)]
     let input: Option<(Box<dyn Read>, String)> = if let Some(path) = args.next() {
         Some((
             Box::new(File::open(&path)?),
@@ -279,7 +293,9 @@ syd::main! {
         }
 
         // SAFETY: create a private, write-sealed copy of the memory-file.
-        let mut fdup = create_memfd(b"syd-sh\0", MFD_ALLOW_SEALING).map(File::from)?;
+        let mut fdup = safe_memfd_create(
+            c"syd-sh",
+            MFdFlags::MFD_ALLOW_SEALING | MFdFlags::MFD_CLOEXEC).map(File::from)?;
         debug!("ctx": "sh",
             "msg": format!("created memory-file {} with sealing allowed",
                 fdup.as_raw_fd()));
