@@ -35,8 +35,9 @@ use crate::{
     },
     confine::{ScmpNotifReq, SydMemoryMap},
     elf::ExecutableFile,
-    fs::{block_signal, retry_on_eintr, sigtimedpoll, unblock_signal, CanonicalPath},
+    fs::{block_signal, sigtimedpoll, unblock_signal, CanonicalPath},
     hash::SydHashMap,
+    retry::retry_on_eintr,
     sigset::SydSigSet,
 };
 
@@ -90,7 +91,6 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Decrement count of handled signals, return true if decremented, false if zero.
-    #[allow(clippy::cognitive_complexity)]
     pub(crate) fn dec_sig_handle(&self, request_tgid: Pid) -> bool {
         let mut is_dec = false;
 
@@ -147,7 +147,6 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Query, remove and return a chdir result.
-    #[allow(clippy::type_complexity)]
     pub(crate) fn get_chdir<'b>(&'b self, pid: Pid) -> Option<(Pid, CanonicalPath<'a>)> {
         self.sysres_map
             .trace_chdir
@@ -166,7 +165,6 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Query, remove and return a error result.
-    #[allow(clippy::type_complexity)]
     pub(crate) fn get_error(&self, pid: Pid) -> Option<(Pid, Option<Errno>)> {
         self.sysres_map
             .trace_error
@@ -176,7 +174,7 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Record a execv result.
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub(crate) fn add_exec(
         &self,
         pid: Pid,
@@ -264,7 +262,6 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Add a blocked syscall.
-    #[allow(clippy::cast_possible_wrap)]
     pub(crate) fn add_sys_block(
         &self,
         request: ScmpNotifReq,
@@ -362,7 +359,6 @@ impl WorkerData {
         Self::total(old_val)
     }
 
-    #[allow(dead_code)]
     pub(crate) fn decrement_worker_total(&self) -> usize {
         let old_val = self.0.fetch_sub(INCREMENT_TOTAL, Ordering::Relaxed);
         Self::total(old_val)
@@ -415,4 +411,10 @@ impl WorkerData {
 //    or mount id so unfortunately we cannot check
 //    for that here.
 // 2. Pid is used for SO_PEERCRED getsockopt(2).
-pub(crate) type UnixMap = Arc<RwLock<SydHashMap<u64, (Pid, Option<UnixAddr>)>>>;
+#[derive(Copy, Clone)]
+pub(crate) struct UnixVal {
+    pub(crate) pid: Pid,
+    pub(crate) addr: Option<UnixAddr>,
+    pub(crate) peer: Option<UnixAddr>,
+}
+pub(crate) type UnixMap = Arc<RwLock<SydHashMap<u64, UnixVal>>>;
